from __future__ import annotations

import typing as t

from audex.exceptions import AudexError
from audex.exceptions import InternalError


class DoctorServiceError(AudexError):
    """Base exception for doctor service errors."""

    default_message = "Doctor service error"
    code: t.ClassVar[int] = 0x30


class InternalDoctorServiceError(InternalError):
    """Internal error in doctor service."""

    default_message = "Internal doctor service error"
    code: t.ClassVar[int] = 0x31


class DoctorNotFoundError(DoctorServiceError):
    """Raised when doctor is not found."""

    __slots__ = ("doctor_id", "message")

    default_message = "Doctor not found"
    code: t.ClassVar[int] = 0x32

    def __init__(self, message: str, *, doctor_id: str) -> None:
        """Initialize the exception.

        Args:
            message: Error message (typically in Chinese for users).
            doctor_id: The ID of the doctor that was not found.
        """
        self.doctor_id = doctor_id
        super().__init__(message)


class InvalidCredentialsError(DoctorServiceError):
    """Raised for invalid login credentials."""

    __slots__ = ("message", "reason")

    default_message = "Invalid credentials"
    code: t.ClassVar[int] = 0x33

    def __init__(self, message: str, *, reason: str | None = None) -> None:
        """Initialize the exception.

        Args:
            message: Error message (typically in Chinese for users).
            reason: Machine-readable reason code (from InvalidCredentialReasons).
        """
        self.reason = reason
        super().__init__(message)


class VoiceprintNotFoundError(DoctorServiceError):
    """Raised when voiceprint is not found."""

    __slots__ = ("doctor_id", "message")

    default_message = "Voiceprint not found"
    code: t.ClassVar[int] = 0x34

    def __init__(self, message: str, *, doctor_id: str) -> None:
        """Initialize the exception.

        Args:
            message: Error message (typically in Chinese for users).
            doctor_id: The ID of the doctor whose voiceprint was not found.
        """
        self.doctor_id = doctor_id
        super().__init__(message)


class DuplicateEIDError(DoctorServiceError):
    """Raised when trying to register with an existing EID."""

    __slots__ = ("eid", "message")

    default_message = "Duplicate EID"
    code: t.ClassVar[int] = 0x35

    def __init__(self, message: str, *, eid: str) -> None:
        """Initialize the exception.

        Args:
            message: Error message (typically in Chinese for users).
            eid: The duplicate employee ID.
        """
        self.eid = eid
        super().__init__(message)



from __future__ import annotations

import typing as t

from audex.exceptions import ConfigurationError
from audex.utils import Unset

if t.TYPE_CHECKING:
    from audex.config import Config
    from audex.lib.transcription import Transcription


def make_transcription(config: Config) -> Transcription:
    if isinstance(config.provider.transcription.dashscope.credential.api_key, Unset):
        raise ConfigurationError(
            config_key="provider.transcription.dashscope.credential.api_key",
            reason="missing",
        )

    from audex.lib.transcription.dashscope import DashscopeParaformer

    return DashscopeParaformer(
        model=config.provider.transcription.dashscope.model,
        api_key=config.provider.transcription.dashscope.credential.api_key,
        user_agent=config.provider.transcription.dashscope.user_agent,
        workspace=config.provider.transcription.dashscope.workspace,
        max_connections=config.provider.transcription.dashscope.websocket.max_connections,
        idle_timeout=config.provider.transcription.dashscope.websocket.idle_timeout,
        drain_timeout=config.provider.transcription.dashscope.websocket.drain_timeout,
        fmt=config.provider.transcription.dashscope.session.fmt,
        sample_rate=config.core.audio.sample_rate,
        silence_duration_ms=config.provider.transcription.dashscope.session.silence_duration_ms,
        vocabulary_id=config.provider.transcription.dashscope.session.vocabulary_id,
        disfluency_removal_enabled=config.provider.transcription.dashscope.session.disfluency_removal_enabled,
        lang_hints=config.provider.transcription.dashscope.session.lang_hints,
        semantic_punctuation=config.provider.transcription.dashscope.session.semantic_punctuation,
        multi_thres_mode=config.provider.transcription.dashscope.session.multi_thres_mode,
        punctuation_pred=config.provider.transcription.dashscope.session.punctuation_pred,
        heartbeat=config.provider.transcription.dashscope.session.heartbeat,
        itn=config.provider.transcription.dashscope.session.itn,
        resources=config.provider.transcription.dashscope.session.resources,
    )



from __future__ import annotations

import sqlmodel as sqlm

from audex.entity.vp import VP
from audex.lib.repos.tables import BaseTable


class VPTable(BaseTable[VP], table=True):
    """Voiceprint (VP) table model for SQLite storage.

    Maps the VP entity to the database table with fields for managing
    voiceprint data associated with doctors.

    Table: vps
    """

    __tablename__ = "vps"

    doctor_id: str = sqlm.Field(
        ...,
        index=True,
        max_length=50,
        description="The ID of the doctor associated with this voiceprint.",
    )
    vpr_uid: str = sqlm.Field(
        ...,
        unique=True,
        index=True,
        max_length=100,
        description="Unique voiceprint recognition UID from the VP service.",
    )
    vpr_group_id: str = sqlm.Field(
        ...,
        index=True,
        max_length=100,
        description="Voiceprint recognition group ID from the VP service.",
    )
    audio_key: str = sqlm.Field(
        ...,
        unique=True,
        index=True,
        max_length=150,
        description="Storage key for the voiceprint audio file.",
    )
    text_content: str = sqlm.Field(
        ...,
        max_length=500,
        description="The text content used for voiceprint enrollment.",
    )
    sample_rate: int = sqlm.Field(
        ...,
        description="Sample rate of the voiceprint audio in Hz.",
    )
    is_active: bool = sqlm.Field(
        default=True,
        description="Indicates whether the voiceprint is active for recognition.",
    )

    @classmethod
    def from_entity(cls, entity: VP) -> VPTable:
        """Create a VPTable instance from a VP entity."""
        return cls(
            id=entity.id,
            doctor_id=entity.doctor_id,
            vpr_uid=entity.vpr_uid,
            vpr_group_id=entity.vpr_group_id,
            audio_key=entity.audio_key,
            text_content=entity.text_content,
            sample_rate=entity.sample_rate,
            is_active=entity.is_active,
            created_at=entity.created_at,
            updated_at=entity.updated_at,
        )

    def to_entity(self) -> VP:
        return VP(
            id=self.id,
            doctor_id=self.doctor_id,
            vpr_uid=self.vpr_uid,
            vpr_group_id=self.vpr_group_id,
            audio_key=self.audio_key,
            text_content=self.text_content,
            is_active=self.is_active,
            sample_rate=self.sample_rate,
            created_at=self.created_at,
            updated_at=self.updated_at,
        )

    def update(self, entity: VP) -> None:
        """Update the VPTable instance with data from a VP entity."""
        self.doctor_id = entity.doctor_id
        self.vpr_uid = entity.vpr_uid
        self.vpr_group_id = entity.vpr_group_id
        self.audio_key = entity.audio_key
        self.text_content = entity.text_content
        self.sample_rate = entity.sample_rate
        self.is_active = entity.is_active
        self.updated_at = entity.updated_at


TABLES: set[type[sqlm.SQLModel]] = {VPTable}



from __future__ import annotations

import abc
import argparse
import pathlib
import typing as t

from pydantic import BaseModel
from pydantic import ConfigDict
from pydantic import ModelWrapValidatorHandler
from pydantic import ValidationError
from pydantic import model_validator

from audex import __description__
from audex import __title__
from audex import __version__
from audex.cli.exceptions import InvalidArgumentError


class BaseArgs(BaseModel, abc.ABC):
    """Base class for CLI arguments with automatic argparse integration.

    Automatically generates argparse arguments from Pydantic fields with support for:
    - Basic types (str, int, float, bool)
    - Optional types (Optional[T])
    - List types (list[T])
    - Literal types for choices
    - Union types
    - Path types
    - Nested subcommands

    Example:
        ```python
        class MyArgs(BaseArgs):
            name: str = Field(description="Your name")
            age: int = Field(default=18, description="Your age")
            verbose: bool = Field(
                default=False, description="Verbose output"
            )

            def run(self) -> None:
                print(f"Hello {self.name}, age {self.age}")


        parser = argparse.ArgumentParser()
        MyArgs.build_args(parser)
        parser.set_defaults(func=MyArgs.func)
        args = parser.parse_args()
        args.func(args)
        ```
    """

    @abc.abstractmethod
    def run(self) -> None:
        """Execute the command logic.

        Must be implemented by subclasses.
        """
        pass

    @classmethod
    def func(cls, args: argparse.Namespace) -> None:
        """Entry point called by argparse.  Parses args and calls run().

        Args:
            args: The argparse Namespace containing parsed arguments.
        """
        instance = cls.parse_args(args)
        instance.run()

    @classmethod
    def build_args(cls, parser: argparse.ArgumentParser) -> None:
        """Build argparse arguments from Pydantic fields.

        Automatically generates appropriate argparse arguments based on field types:
        - bool: Uses store_true/store_false based on default value
        - list[T]: Uses nargs='+' or nargs='*'
        - Optional[T]: Makes argument optional with default None
        - Literal[... ]: Uses choices parameter
        - Path/pathlib.Path: Uses type=pathlib.Path

        Args:
            parser: The argparse parser to add arguments to.
        """
        for field, fieldinfo in cls.__pydantic_fields__.items():
            # Skip internal fields
            if field.startswith("_"):
                continue

            arg_name = f"--{field.replace('_', '-')}"
            alias = fieldinfo.alias
            flags = (arg_name, f"-{alias}") if alias else (arg_name,)
            required = fieldinfo.is_required()
            ann = fieldinfo.annotation
            help_text = fieldinfo.description or ""
            default = fieldinfo.default

            # Unwrap the annotation to get the actual type
            origin = t.get_origin(ann)
            args_types = t.get_args(ann)

            # Handle Optional[T] (Union[T, None])
            if origin is t.Union:
                # Filter out NoneType
                non_none_types = [a for a in args_types if a is not type(None)]
                if type(None) in args_types and non_none_types:
                    # This is Optional[T]
                    ann = (
                        non_none_types[0]
                        if len(non_none_types) == 1
                        else t.Union[tuple(non_none_types)]  # noqa
                    )
                    required = False
                    origin = t.get_origin(ann)
                    args_types = t.get_args(ann)

            # Handle Literal types
            choices = None
            if origin is t.Literal:
                choices = list(args_types)
                ann = type(choices[0]) if choices else str
                origin = None  # Reset origin since we've extracted the type

            # Handle List types
            if origin is list:
                item_type = args_types[0] if args_types else str

                # Handle List[Literal[...]]
                if t.get_origin(item_type) is t.Literal:
                    choices = list(t.get_args(item_type))
                    item_type = type(choices[0]) if choices else str

                # Determine nargs
                nargs = "*" if not required else "+"

                parser.add_argument(
                    *flags,
                    type=item_type,
                    nargs=nargs,
                    default=default if not required else None,
                    help=help_text,
                    choices=choices,
                    dest=field,
                )
                continue

            # Handle Path types
            if ann in (pathlib.Path, pathlib.PosixPath, pathlib.WindowsPath):
                if required:
                    parser.add_argument(
                        *flags,
                        required=True,
                        type=pathlib.Path,
                        help=help_text,
                        dest=field,
                    )
                else:
                    parser.add_argument(
                        *flags,
                        type=pathlib.Path,
                        default=default,
                        help=help_text,
                        dest=field,
                    )
                continue

            # Handle boolean types
            is_bool = ann is bool or (origin is None and ann is bool)
            if is_bool:
                # Determine action based on default value
                if default is True:
                    # If default is True, use store_false with --no-prefix
                    neg_flags = (
                        (f"--no-{field.replace('_', '-')}", f"-N{alias}")
                        if alias
                        else (f"--no-{field.replace('_', '-')}",)
                    )
                    parser.add_argument(
                        *neg_flags,
                        action="store_false",
                        help=help_text or f"Disable {field}",
                        dest=field,
                        default=True,
                    )
                else:
                    # If default is False or not set, use store_true
                    parser.add_argument(
                        *flags,
                        action="store_true",
                        help=help_text,
                        dest=field,
                        default=False,
                    )
                continue

            # Handle regular types (str, int, float, etc.)
            try:
                type_func = ann if callable(ann) else str
            except Exception:
                type_func = str

            if required:
                parser.add_argument(
                    *flags,
                    required=True,
                    type=type_func,
                    help=help_text,
                    choices=choices,
                    dest=field,
                )
            else:
                parser.add_argument(
                    *flags,
                    default=default,
                    type=type_func,
                    help=help_text,
                    choices=choices,
                    dest=field,
                )

    @classmethod
    def parse_args(cls, args: argparse.Namespace) -> t.Self:
        """Parse argparse Namespace into a Pydantic model instance.

        Args:
            args: The argparse Namespace to parse.

        Returns:
            An instance of the class with validated fields.

        Raises:
            pydantic.ValidationError: If validation fails.
        """
        # Convert Namespace to dict and filter out None values for optional fields
        args_dict = vars(args)

        # Remove non-field attributes (like 'func')
        field_names = set(cls.__pydantic_fields__.keys())
        filtered_dict = {k: v for k, v in args_dict.items() if k in field_names}

        return cls.model_validate(filtered_dict, by_alias=False, by_name=True, strict=False)

    @classmethod
    def register_subparser(
        cls,
        subparsers: argparse._SubParsersAction,
        name: str,
        help_text: str | None = None,
        aliases: list[str] | None = None,
    ) -> argparse.ArgumentParser:
        """Register this command as a subcommand.

        Supports nested subcommands by returning the parser which can have
        its own subparsers added.

        Args:
            subparsers: The subparsers action from parent parser.
            name: The name of this subcommand.
            help_text: Help text for this subcommand.  If None, uses class docstring.
            aliases: Alternative names for this subcommand.

        Returns:
            The argument parser for this subcommand, which can be used to
            add nested subcommands.

        Example:
            ```python
            # Create main parser
            parser = argparse.ArgumentParser()
            subparsers = parser.add_subparsers(
                dest="command", required=True
            )

            # Register top-level command
            db_parser = DBCommand.register_subparser(
                subparsers, "db", "Database commands"
            )

            # Add nested subcommands
            db_subparsers = db_parser.add_subparsers(
                dest="db_command", required=True
            )
            MigrateCommand.register_subparser(
                db_subparsers, "migrate", "Run migrations"
            )
            ```
        """
        if help_text is None:
            help_text = cls.__doc__.strip() if cls.__doc__ else f"{name} command"

        parser = subparsers.add_parser(
            name,
            help=help_text,
            description=help_text,
            aliases=aliases or [],
        )
        cls.build_args(parser)
        parser.set_defaults(func=cls.func)
        return parser

    @model_validator(mode="wrap")
    @classmethod
    def reraise(cls, data: t.Any, handler: ModelWrapValidatorHandler[t.Self]) -> t.Self:
        """Reraise validation errors with clearer messages.

        Args:
            data: The input data to validate.
            handler: The model wrap validator handler.

        Returns:
            The validated model instance.

        Raises:
            InvalidArgumentError: If validation fails.
        """
        try:
            return handler(data)
        except ValidationError as e:
            raise InvalidArgumentError.from_validation_error(e) from e

    model_config = ConfigDict(
        extra="ignore",
        frozen=True,
        arbitrary_types_allowed=True,  # Allow types like pathlib.Path
        populate_by_name=True,  # Allow population by field name
    )


def parser_with_version(
    prog: str = __title__,
    description: str = __description__,
) -> argparse.ArgumentParser:
    """Create an argument parser with --version and --help built-in.

    Helper function to create a parser with common options.

    Args:
        prog: Program name.
        description: Program description.

    Returns:
        Configured ArgumentParser with version info.
    """
    parser = argparse.ArgumentParser(
        prog=prog,
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--version",
        "-V",
        action="version",
        version=f"%(prog)s {__version__}",
        help="Show the version number and exit.",
    )
    return parser



from __future__ import annotations

import io
import json
import pathlib
import zipfile

from audex.entity.segment import Segment
from audex.entity.session import Session
from audex.entity.utterance import Utterance
from audex.filters.generated import segment_filter
from audex.filters.generated import utterance_filter
from audex.helper.mixin import LoggingMixin
from audex.lib.repos.segment import SegmentRepository
from audex.lib.repos.session import SessionRepository
from audex.lib.repos.utterance import UtteranceRepository
from audex.lib.server.types import AudioMetadataItem
from audex.lib.server.types import AudioMetadataJSON
from audex.lib.server.types import ConversationJSON
from audex.lib.server.types import SegmentDict
from audex.lib.server.types import SessionDict
from audex.lib.server.types import SessionExportData
from audex.lib.server.types import UtteranceDict
from audex.lib.store import Store


class Exporter(LoggingMixin):
    """Exporter for packaging session data and audio files."""

    __logtag__ = "audex.lib.exporter"

    def __init__(
        self,
        session_repo: SessionRepository,
        segment_repo: SegmentRepository,
        utterance_repo: UtteranceRepository,
        store: Store,
    ):
        super().__init__()
        self.session_repo = session_repo
        self.segment_repo = segment_repo
        self.utterance_repo = utterance_repo
        self.store = store

    async def export_session_data(self, session_id: str) -> SessionExportData:
        """Export session data as structured format."""
        # Get session
        session = await self.session_repo.read(session_id)
        if not session:
            raise ValueError(f"Session {session_id} not found")

        # Get utterances
        utt_filter = utterance_filter().session_id.eq(session_id).sequence.asc()
        utterances = await self.utterance_repo.list(utt_filter.build())

        # Get segments
        seg_filter = segment_filter().session_id.eq(session_id).sequence.asc()
        segments = await self.segment_repo.list(seg_filter.build())

        # Convert to typed dicts
        return SessionExportData(
            session=self._session_to_dict(session),
            utterances=[self._utterance_to_dict(u) for u in utterances],
            segments=[self._segment_to_dict(s) for s in segments],
        )

    async def export_session_zip(self, session_id: str) -> bytes:
        """Export session as ZIP package."""
        export_data = await self.export_session_data(session_id)
        zip_buffer = io.BytesIO()

        with zipfile.ZipFile(zip_buffer, "w", zipfile.ZIP_DEFLATED) as zipf:
            # Add conversation.json
            conversation: ConversationJSON = {
                "session": export_data["session"],
                "utterances": export_data["utterances"],
                "total_utterances": len(export_data["utterances"]),
                "total_segments": len(export_data["segments"]),
            }
            zipf.writestr(
                "conversation.json",
                json.dumps(conversation, ensure_ascii=False, indent=2),
            )

            # Add audio files
            audio_metadata_items: list[AudioMetadataItem] = []

            for idx, segment_dict in enumerate(export_data["segments"], start=1):
                audio_key = segment_dict["audio_key"]

                try:
                    audio_data = await self.store.download(audio_key)
                    ext = pathlib.Path(audio_key).suffix or ".mp3"
                    filename = f"segment_{idx:03d}{ext}"

                    zipf.writestr(f"audio/{filename}", audio_data)

                    audio_metadata_items.append(
                        AudioMetadataItem(
                            filename=filename,
                            sequence=segment_dict["sequence"],
                            duration_ms=segment_dict["duration_ms"],
                            started_at=segment_dict["started_at"],
                            ended_at=segment_dict["ended_at"],
                        )
                    )

                    self.logger.debug(f"Added audio file: {filename}")

                except Exception as e:
                    self.logger.error(f"Failed to add audio {audio_key}: {e}")

            # Add audio metadata
            if audio_metadata_items:
                audio_metadata: AudioMetadataJSON = {
                    "session_id": session_id,
                    "total_segments": len(audio_metadata_items),
                    "segments": audio_metadata_items,
                }
                zipf.writestr(
                    "audio/metadata. json",
                    json.dumps(audio_metadata, ensure_ascii=False, indent=2),
                )

        zip_buffer.seek(0)
        return zip_buffer.getvalue()

    async def export_multiple_sessions_zip(self, session_ids: list[str]) -> bytes:
        """Export multiple sessions as ZIP package."""
        zip_buffer = io.BytesIO()

        with zipfile.ZipFile(zip_buffer, "w", zipfile.ZIP_DEFLATED) as zipf:
            for session_id in session_ids:
                try:
                    session_zip_data = await self.export_session_zip(session_id)

                    with zipfile.ZipFile(io.BytesIO(session_zip_data), "r") as session_zipf:
                        for file_info in session_zipf.infolist():
                            file_data = session_zipf.read(file_info.filename)
                            new_path = f"{session_id}/{file_info.filename}"
                            zipf.writestr(new_path, file_data)

                    self.logger.info(f"Added session {session_id} to export")

                except Exception as e:
                    self.logger.error(f"Failed to export session {session_id}: {e}")

        zip_buffer.seek(0)
        return zip_buffer.getvalue()

    def _session_to_dict(self, session: Session) -> SessionDict:
        """Convert Session to typed dict."""
        return SessionDict(
            id=session.id,
            doctor_id=session.doctor_id,
            patient_name=session.patient_name,
            clinic_number=session.clinic_number,
            medical_record_number=session.medical_record_number,
            diagnosis=session.diagnosis,
            status=session.status.value,
            started_at=session.started_at.isoformat() if session.started_at else None,
            ended_at=session.ended_at.isoformat() if session.ended_at else None,
            created_at=session.created_at.isoformat(),
        )

    def _utterance_to_dict(self, utterance: Utterance) -> UtteranceDict:
        """Convert Utterance to typed dict."""
        return UtteranceDict(
            id=utterance.id,
            sequence=utterance.sequence,
            speaker=utterance.speaker.value,
            text=utterance.text,
            confidence=utterance.confidence,
            start_time_ms=utterance.start_time_ms,
            end_time_ms=utterance.end_time_ms,
            duration_ms=utterance.duration_ms,
            timestamp=utterance.timestamp.isoformat(),
        )

    def _segment_to_dict(self, segment: Segment) -> SegmentDict:
        """Convert Segment to typed dict."""
        return SegmentDict(
            id=segment.id,
            sequence=segment.sequence,
            audio_key=segment.audio_key,
            started_at=segment.started_at.isoformat(),
            ended_at=segment.ended_at.isoformat() if segment.ended_at else None,
            duration_ms=segment.duration_ms,
        )



from __future__ import annotations

import abc
import typing as t

from audex.helper.mixin import AsyncContextMixin


class Database(AsyncContextMixin, abc.ABC):
    """Abstract base class for database containers.

    All database implementations should inherit from this class and implement
    the required abstract methods. This ensures a consistent interface across
    different database backends.

    The class provides:
    1. Lifecycle management through AsyncContextMixin (init/close)
    2. Health check interface (ping)
    3. Raw query execution interface (exec)

    Example:
        ```python
        class MyDatabase(Database):
            async def init(self) -> None:
                # Initialize connection
                pass

            async def close(self) -> None:
                # Close connection
                pass

            async def ping(self) -> bool:
                # Check connectivity
                pass

            async def exec(
                self, readonly: bool, **kwargs: t.Any
            ) -> t.Any:
                # Execute raw query
                pass
        ```
    """

    @abc.abstractmethod
    async def ping(self) -> bool:
        """Check database connectivity.

        Returns:
            True if database is reachable and healthy, False otherwise.

        Note:
            Implementations should not raise exceptions. Instead, catch
            any errors and return False.
        """
        pass

    @abc.abstractmethod
    async def exec(self, *args: t.Any, **kwargs: t.Any) -> t.Any:
        """Execute a raw database command.

        This method provides a way to execute native database commands
        when ORM abstractions are insufficient or when optimization
        requires direct database access.

        Args:
            *args: Positional arguments for the database command.
            **kwargs: Database-specific command parameters.

        Returns:
            Database-specific result object.

        Raises:
            RuntimeError: If the execution fails.

        Note:
            The exact signature and return type will vary by database
            implementation. Refer to specific database class documentation
            for details.
        """
        pass

    def __repr__(self) -> str:
        return f"DATABASE <{self.__class__.__name__}>"



from __future__ import annotations

import typing as t

from audex.valueobj.common.ops import Op
from audex.valueobj.common.ops import Order

if t.TYPE_CHECKING:
    from audex.entity import Entity
    from audex.entity.fields import FieldSpec

E = t.TypeVar("E", bound="Entity")
T = t.TypeVar("T")


class SortSpec:
    """Sort specification for a single field.

    Attributes:
        field: The field name to sort by.
        order: The sort order (ASC or DESC).
    """

    field: str
    order: Order

    __slots__ = ("field", "order")

    def __init__(self, field: str, order: Order = Order.ASC) -> None:
        object.__setattr__(self, "field", field)
        object.__setattr__(self, "order", order)

    def __setattr__(self, key: str, value: t.Any) -> None:
        raise AttributeError("SortSpec instances are immutable")

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, SortSpec):
            return NotImplemented
        return self.field == other.field and self.order == other.order

    def __hash__(self) -> int:
        return hash((self.field, self.order))

    def __repr__(self) -> str:
        return f"SortSpec(field={self.field!r}, order={self.order.value})"

    def __str__(self) -> str:
        direction = "↑" if self.order == Order.ASC else "↓"
        return f"{self.field} {direction}"


class ConditionSpec:
    """Immutable filter condition.

    Represents a single filter condition with field name, operation, and
    value(s).

    Attributes:
        field: The field name the condition applies to.
        op: The operation (from Op enum).
        value: The value to compare against.
        value2: The second value for operations like BETWEEN (optional).
    """

    field: str
    op: Op
    value: object
    value2: object | None

    __slots__ = ("field", "op", "value", "value2")

    def __init__(self, field: str, op: Op, value: object, value2: object | None = None) -> None:
        object.__setattr__(self, "field", field)
        object.__setattr__(self, "op", op)
        object.__setattr__(self, "value", value)
        object.__setattr__(self, "value2", value2)

    def __setattr__(self, key: str, value: t.Any) -> None:
        raise AttributeError("Condition instances are immutable")

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, ConditionSpec):
            return NotImplemented
        return (
            self.field == other.field
            and self.op == other.op
            and self.value == other.value
            and self.value2 == other.value2
        )

    def __hash__(self) -> int:
        return hash((self.field, self.op, self.value, self.value2))

    def __repr__(self) -> str:
        return (
            f"Condition(field={self.field!r}, op={self.op!r}, "
            f"value={self.value!r}, value2={self.value2!r})"
        )

    def __str__(self) -> str:
        if self.value2 is not None:
            return f"{self.field} {self.op.name} {self.value}, {self.value2}"
        return f"{self.field} {self.op.name} {self.value}"


class ConditionGroup:
    """Group of conditions with AND/OR logic.

    Attributes:
        conditions: List of conditions or nested groups.
        operator: "AND" or "OR" - how to combine conditions.
        negated: Whether to negate the entire group.
    """

    __slots__ = ("conditions", "negated", "operator")

    def __init__(
        self,
        conditions: list[ConditionSpec | ConditionGroup] | None = None,
        operator: t.Literal["AND", "OR"] = "AND",
        negated: bool = False,
    ) -> None:
        self.conditions: list[ConditionSpec | ConditionGroup] = conditions or []
        self.operator = operator
        self.negated = negated

    def add(self, condition: ConditionSpec | ConditionGroup) -> None:
        """Add a condition or group to this group."""
        self.conditions.append(condition)

    def __repr__(self) -> str:
        return f"ConditionGroup(operator={self.operator}, conditions={self.conditions})"

    def __str__(self) -> str:
        if not self.conditions:
            return "(empty)"

        inner = f" {self.operator} ".join(
            f"({c})" if isinstance(c, ConditionGroup) else str(c) for c in self.conditions
        )
        s = f"({inner})"
        return f"NOT {s}" if self.negated else s


class Filter:
    """Container for filter conditions with AND/OR support.

    Attributes:
        _condition_group: Root condition group (can contain nested groups).
        _sorts: List of sort specifications.
        _entity_class: The entity class this filter applies to.
    """

    __slots__ = ("_builder", "_condition_group", "_entity_class", "_sorts")

    def __init__(
        self,
        entity_class: type[Entity],
        builder: FilterBuilder[t.Any] | None = None,
    ) -> None:
        self._condition_group = ConditionGroup(operator="AND")
        self._sorts: list[SortSpec] = []
        self._entity_class = entity_class
        self._builder = builder

    @property
    def condition_group(self) -> ConditionGroup:
        """Get the root condition group."""
        return self._condition_group

    @property
    def sorts(self) -> list[SortSpec]:
        """Get the list of sort specifications."""
        return self._sorts

    @property
    def entity_class(self) -> type[Entity]:
        """Get the entity class this filter applies to."""
        return self._entity_class

    @property
    def conditions(self) -> list[ConditionSpec]:
        """Legacy property for backward compatibility.

        Returns flat list of conditions from the root AND group.
        Warning: This loses OR grouping information!
        """
        return [c for c in self._condition_group.conditions if isinstance(c, ConditionSpec)]

    def _add_condition(self, condition: ConditionSpec) -> t.Self:
        """Add a condition to the current filter (AND logic).

        Args:
            condition: The condition to add.

        Returns:
            Self for method chaining.
        """
        self._condition_group.add(condition)
        return self

    def _add_sort(self, sort: SortSpec) -> t.Self:
        """Add a sort specification to the filter.

        Args:
            sort: The sort specification to add.

        Returns:
            Self for method chaining.
        """
        self._sorts.append(sort)
        return self

    def or_(self, *filters: Filter) -> t.Self:
        """Combine conditions with OR logic.

        Creates a new OR group containing:
        - Current filter's conditions
        - All provided filters' conditions

        Args:
            *filters: Other filters to combine with OR logic.

        Returns:
            Self for method chaining.

        Example:
            ```python
            # Find users with username "john" OR email "john@example.com"
            filter1 = user_filter().username.eq("john")
            filter2 = user_filter().email.eq("john@example.com")
            combined = filter1.or_(filter2)

            # Or using method chaining:
            combined = (
                user_filter()
                .username.eq("john")
                .or_(user_filter().email.eq("john@example.com"))
            )
            ```
        """
        # Validate same entity type
        for f in filters:
            if self._entity_class != f._entity_class:
                raise ValueError(
                    f"Cannot combine filters for different entity types: "
                    f"{self._entity_class.__name__} and {f._entity_class.__name__}"
                )

        # Create new OR group
        or_group = ConditionGroup(operator="OR")

        # Add current conditions
        if self._condition_group.conditions:
            if len(self._condition_group.conditions) == 1:
                or_group.add(self._condition_group.conditions[0])
            else:
                # Wrap in AND group if multiple conditions
                or_group.add(
                    ConditionGroup(
                        conditions=self._condition_group.conditions.copy(), operator="AND"
                    )
                )

        # Add other filters' conditions
        for f in filters:
            if f._condition_group.conditions:
                if len(f._condition_group.conditions) == 1:
                    or_group.add(f._condition_group.conditions[0])
                else:
                    or_group.add(
                        ConditionGroup(
                            conditions=f._condition_group.conditions.copy(), operator="AND"
                        )
                    )

        # Replace root group with OR group
        self._condition_group = or_group

        # Merge sorts (keep unique)
        for f in filters:
            for sort in f._sorts:
                if sort not in self._sorts:
                    self._sorts.append(sort)

        return self

    def not_(self) -> Filter:
        """Mark this filter as negated (NOT)."""
        new_filter = Filter(self._entity_class, self._builder)
        new_filter._condition_group = ConditionGroup(
            conditions=self._condition_group.conditions.copy(),
            operator=self._condition_group.operator,
            negated=True,
        )
        new_filter._sorts = self._sorts.copy()
        return new_filter

    def __getattr__(self, name: str) -> FieldFilter[t.Any]:
        """Allow chaining through the original builder.

        This enables: filter().field1.eq(x).field2.eq(y)
        """
        if name.startswith("_"):
            raise AttributeError(f"'{type(self).__name__}' has no attribute '{name}'")

        # Delegate to the builder if available
        if self._builder is not None:
            return getattr(self._builder, name)  # type: ignore

        # Fallback: check if field exists
        if name not in self._entity_class._fields:
            raise AttributeError(f"Entity '{self._entity_class.__name__}' has no field '{name}'")

        field = self._entity_class._fields[name]

        from audex.entity.fields import ListFieldSpec as ListField
        from audex.entity.fields import StringBackedFieldSpec as StringBackedField
        from audex.entity.fields import StringFieldSpec as StringField

        if isinstance(field, StringField):
            return StringFieldFilter(name, self)
        if isinstance(field, StringBackedField):
            return StringBackedFieldFilter(name, self)
        if isinstance(field, ListField):
            return ListFieldFilter(name, self)

        return FieldFilter(name, self)

    def __and__(self, other: Filter) -> Filter:
        """Combine two filters with AND logic.

        Args:
            other: Another filter to combine.

        Returns:
            A new filter with combined conditions.

        Raises:
            ValueError: If filters are for different entity types.
        """
        if self._entity_class != other._entity_class:
            raise ValueError(
                f"Cannot combine filters for different entity types: "
                f"{self._entity_class.__name__} and {other._entity_class.__name__}"
            )

        new_filter = Filter(self._entity_class, self._builder)

        # Combine condition groups with AND
        new_filter._condition_group = ConditionGroup(operator="AND")

        # Add self's conditions
        if self._condition_group.conditions:
            if (
                len(self._condition_group.conditions) == 1
                and self._condition_group.operator == "AND"
            ):
                new_filter._condition_group.add(self._condition_group.conditions[0])
            else:
                new_filter._condition_group.add(self._condition_group)

        # Add other's conditions
        if other._condition_group.conditions:
            if (
                len(other._condition_group.conditions) == 1
                and other._condition_group.operator == "AND"
            ):
                new_filter._condition_group.add(other._condition_group.conditions[0])
            else:
                new_filter._condition_group.add(other._condition_group)

        # Merge sorts
        new_filter._sorts = self._sorts + other._sorts

        return new_filter

    def __or__(self, other: Filter) -> Filter:
        """Combine two filters with OR logic (operator overload).

        Args:
            other: Another filter to combine.

        Returns:
            A new filter with OR combination.

        Example:
            ```python
            filter1 = user_filter().username.eq("john")
            filter2 = user_filter().email.eq("john@example.com")
            combined = filter1 | filter2  # Using | operator
            ```
        """
        if self._entity_class != other._entity_class:
            raise ValueError(
                f"Cannot combine filters for different entity types: "
                f"{self._entity_class.__name__} and {other._entity_class.__name__}"
            )

        new_filter = Filter(self._entity_class, self._builder)
        new_filter._condition_group = ConditionGroup(operator="OR")

        # Add self's conditions
        if self._condition_group.conditions:
            if len(self._condition_group.conditions) == 1:
                new_filter._condition_group.add(self._condition_group.conditions[0])
            else:
                new_filter._condition_group.add(
                    ConditionGroup(
                        conditions=self._condition_group.conditions.copy(),
                        operator=self._condition_group.operator,
                    )
                )

        # Add other's conditions
        if other._condition_group.conditions:
            if len(other._condition_group.conditions) == 1:
                new_filter._condition_group.add(other._condition_group.conditions[0])
            else:
                new_filter._condition_group.add(
                    ConditionGroup(
                        conditions=other._condition_group.conditions.copy(),
                        operator=other._condition_group.operator,
                    )
                )

        # Merge sorts
        new_filter._sorts = self._sorts + other._sorts

        return new_filter

    def __invert__(self) -> Filter:
        """Negate the filter (NOT).

        Returns:
            A new filter that is the negation of this filter.
        """
        new_filter = Filter(self._entity_class, self._builder)
        new_filter._condition_group = ConditionGroup(
            conditions=self._condition_group.conditions.copy(),
            operator=self._condition_group.operator,
            negated=True,
        )
        new_filter._sorts = self._sorts.copy()
        return new_filter

    def __repr__(self) -> str:
        return (
            f"FILTER<{self._entity_class.__name__}>({self._condition_group}, sorts={self._sorts})"
        )


class FieldFilter(t.Generic[T]):
    """Type-safe field filter builder.

    Provides comparison methods that return the parent Filter for
    chaining.
    """

    __slots__ = ("_field_name", "_filter")

    def __init__(self, field_name: str, filter_obj: Filter) -> None:
        self._field_name = field_name
        self._filter = filter_obj

    def eq(self, value: T) -> Filter:
        """Field equals value."""
        self._filter._add_condition(ConditionSpec(self._field_name, Op.EQ, value))
        return self._filter

    def ne(self, value: T) -> Filter:
        """Field not equals value."""
        self._filter._add_condition(ConditionSpec(self._field_name, Op.NE, value))
        return self._filter

    def gt(self, value: T) -> Filter:
        """Field greater than value."""
        self._filter._add_condition(ConditionSpec(self._field_name, Op.GT, value))
        return self._filter

    def lt(self, value: T) -> Filter:
        """Field less than value."""
        self._filter._add_condition(ConditionSpec(self._field_name, Op.LT, value))
        return self._filter

    def gte(self, value: T) -> Filter:
        """Field greater than or equal to value."""
        self._filter._add_condition(ConditionSpec(self._field_name, Op.GTE, value))
        return self._filter

    def lte(self, value: T) -> Filter:
        """Field less than or equal to value."""
        self._filter._add_condition(ConditionSpec(self._field_name, Op.LTE, value))
        return self._filter

    def in_(self, values: t.Sequence[T]) -> Filter:
        """Field in list of values."""
        self._filter._add_condition(ConditionSpec(self._field_name, Op.IN, values))
        return self._filter

    def nin(self, values: t.Sequence[T]) -> Filter:
        """Field not in list of values."""
        self._filter._add_condition(ConditionSpec(self._field_name, Op.NIN, values))
        return self._filter

    def between(self, value1: T, value2: T) -> Filter:
        """Field between two values (inclusive)."""
        self._filter._add_condition(ConditionSpec(self._field_name, Op.BETWEEN, value1, value2))
        return self._filter

    def is_null(self) -> Filter:
        """Field is NULL/None."""
        self._filter._add_condition(ConditionSpec(self._field_name, Op.EQ, None))
        return self._filter

    def is_not_null(self) -> Filter:
        """Field is not NULL/None."""
        self._filter._add_condition(ConditionSpec(self._field_name, Op.NE, None))
        return self._filter

    def asc(self) -> Filter:
        """Sort field in ascending order."""
        self._filter._add_sort(SortSpec(self._field_name, Order.ASC))
        return self._filter

    def desc(self) -> Filter:
        """Sort field in descending order."""
        self._filter._add_sort(SortSpec(self._field_name, Order.DESC))
        return self._filter

    def __repr__(self) -> str:
        return f"FieldFilter<{self._field_name}>()"


class StringFieldFilter(FieldFilter[str]):
    """String-specific field filter with additional operations."""

    def contains(self, value: str) -> Filter:
        """Field contains substring."""
        self._filter._add_condition(ConditionSpec(self._field_name, Op.CONTAINS, value))
        return self._filter

    def startswith(self, value: str) -> Filter:
        """Field starts with substring (uses ^prefix pattern)."""
        self._filter._add_condition(ConditionSpec(self._field_name, Op.CONTAINS, f"^{value}"))
        return self._filter

    def endswith(self, value: str) -> Filter:
        """Field ends with substring (uses suffix$ pattern)."""
        self._filter._add_condition(ConditionSpec(self._field_name, Op.CONTAINS, f"{value}$"))
        return self._filter


class StringBackedFieldFilter(FieldFilter[T]):
    """Filter for fields that are persisted as strings but have custom
    types.

    Supports string operations like contains, startswith, endswith.
    """

    def contains(self, value: str) -> Filter:
        """Field contains substring."""
        self._filter._add_condition(ConditionSpec(self._field_name, Op.CONTAINS, value))
        return self._filter

    def startswith(self, value: str) -> Filter:
        """Field starts with substring (uses ^prefix pattern)."""
        self._filter._add_condition(ConditionSpec(self._field_name, Op.CONTAINS, f"^{value}"))
        return self._filter

    def endswith(self, value: str) -> Filter:
        """Field ends with substring (uses suffix$ pattern)."""
        self._filter._add_condition(ConditionSpec(self._field_name, Op.CONTAINS, f"{value}$"))
        return self._filter


class ListFieldFilter(FieldFilter[list[T]]):
    """Filter for list/collection fields.

    Supports contains operation to check if a value exists in the list.
    """

    def has(self, value: T) -> Filter:
        """Check if list contains value."""
        self._filter._add_condition(ConditionSpec(self._field_name, Op.HAS, value))
        return self._filter


class FilterBuilder(t.Generic[E]):
    """Type-safe filter builder for entities.

    This class dynamically creates properties for each field in the entity,
    allowing type-safe filter construction with IDE autocomplete.

    Example:
        ```python
        # Single condition
        filter1 = User.filter().username.eq("john")

        # Multiple conditions (AND - chained)
        filter2 = (
            User.filter()
            .username.contains("test")
            .is_active.eq(True)
            .tier.in_([UserTier.PREMIUM, UserTier.VIP])
        )

        # OR combination - Method 1: or_() method
        filter3 = (
            User.filter()
            .username.eq("john")
            .or_(User.filter().email.eq("john@example.com"))
        )

        # OR combination - Method 2: | operator
        filter4 = User.filter().username.eq(
            "john"
        ) | User.filter().email.eq("john@example.com")

        # Complex: (username = "john" OR email = "john@ex.com") AND is_active = True
        filter5 = (
            User.filter().username.eq("john")
            | User.filter().email.eq("john@example.com")
        ) & User.filter().is_active.eq(True)

        # Combining filters with &
        active_filter = User.filter().is_active.eq(True)
        premium_filter = User.filter().tier.eq(UserTier.PREMIUM)
        combined = active_filter & premium_filter
        ```
    """

    __slots__ = ("_entity_class", "_filter")

    def __init__(self, entity_class: type[E]) -> None:
        object.__setattr__(self, "_entity_class", entity_class)
        # Pass self to Filter so it can delegate back to us
        filter_obj = Filter(entity_class, builder=self)
        object.__setattr__(self, "_filter", filter_obj)

    def __getattr__(self, name: str) -> FieldFilter[t.Any]:
        """Dynamically create field filters for entity fields.

        Args:
            name: The field name to filter on.

        Returns:
            A FieldFilter for the requested field.

        Raises:
            AttributeError: If the field doesn't exist in the entity.
        """
        if name.startswith("_"):
            raise AttributeError(f"'{type(self).__name__}' has no attribute '{name}'")

        entity_class: type[E] = object.__getattribute__(self, "_entity_class")
        if name not in entity_class._fields:
            raise AttributeError(f"Entity '{entity_class.__name__}' has no field '{name}'")

        field: FieldSpec[t.Any] = entity_class._fields[name]
        filter_obj: Filter = object.__getattribute__(self, "_filter")

        # Return appropriate filter type based on field type
        from audex.entity.fields import ListFieldSpec as ListField
        from audex.entity.fields import StringBackedFieldSpec as StringBackedField
        from audex.entity.fields import StringFieldSpec as StringField

        if isinstance(field, StringField):
            return StringFieldFilter(name, filter_obj)
        if isinstance(field, StringBackedField):
            return StringBackedFieldFilter(name, filter_obj)
        if isinstance(field, ListField):
            return ListFieldFilter(name, filter_obj)

        return FieldFilter(name, filter_obj)

    def __setattr__(self, name: str, value: t.Any) -> None:
        """Prevent attribute assignment to maintain immutability."""
        raise AttributeError("FilterBuilder attributes cannot be modified")

    def build(self) -> Filter:
        """Build and return the final Filter object.

        Returns:
            The constructed Filter with all conditions.

        Note:
            This method is optional. The Filter is automatically returned
            after each condition method call, so you can use the filter
            directly without calling build().
        """
        return object.__getattribute__(self, "_filter")  # type: ignore



from __future__ import annotations

import random

from dependency_injector.wiring import Provide
from dependency_injector.wiring import inject
from fastapi import Depends
from nicegui import ui

from audex.container import Container
from audex.service.doctor import DoctorService
from audex.service.export import ExportService
from audex.service.session import SessionService
from audex.view.decorators import handle_errors


@ui.page("/sessions/export")
@handle_errors
@inject
async def render(
    doctor_service: DoctorService = Depends(Provide[Container.service.doctor]),
    session_service: SessionService = Depends(Provide[Container.service.session]),
    export_service: ExportService = Depends(Provide[Container.service.export]),
) -> None:
    """Render export options page."""

    # Get current doctor
    doctor = await doctor_service.current_doctor()

    # Add CSS
    ui.add_head_html('<link rel="stylesheet" href="/static/css/sessions/styles.css">')

    # Fetch sessions
    sessions = await session_service.list(doctor_id=doctor.id, page_size=100)

    # State
    server_running = {"value": False}

    # Before unload script
    ui.add_head_html("""
    <script>
        window.addEventListener('beforeunload', function(e) {
            if (window.serverRunning) {
                e.preventDefault();
                e.returnValue = '服务器正在运行，确定要离开吗？';
                return e.returnValue;
            }
        });
    </script>
    """)

    # Header
    with (
        ui.header().classes("header-glass items-center justify-between px-6 py-3"),
        ui.row().classes("items-center gap-3"),
    ):

        async def go_back():
            """返回并检查服务器状态."""
            if server_running["value"]:
                with (
                    ui.dialog() as leave_dialog,
                    ui.card()
                    .classes("dialog-card")
                    .style("width: 450px; padding: 28px; border-radius: 16px;"),
                ):
                    with ui.row().classes("w-full items-center mb-6"):
                        ui.icon("warning", size="xl").classes("text-warning")
                        ui.label("确认离开").classes("text-h5 font-bold text-grey-9 ml-3 flex-1")
                        ui.button(icon="close", on_click=leave_dialog.close).props(
                            "flat round dense"
                        )

                    ui.label("服务器正在运行，离开将自动关闭服务器").classes(
                        "text-body1 text-grey-8 mb-2"
                    )
                    ui.label("确定要离开吗？").classes("text-body2 text-grey-7 mb-6")

                    with ui.row().classes("w-full gap-3 justify-end"):
                        ui.button("取消", on_click=leave_dialog.close).props(
                            "outline color=grey-8 no-caps"
                        ).classes("action-button")

                        async def confirm_leave():
                            await export_service.stop_server()
                            server_running["value"] = False
                            await ui.run_javascript("window.serverRunning = false;")
                            leave_dialog.close()
                            ui.navigate.to("/sessions")

                        ui.button("确认离开", on_click=confirm_leave).props(
                            "unelevated color=negative no-caps"
                        ).classes("action-button")

                leave_dialog.open()
            else:
                ui.navigate.to("/sessions")

        ui.button(icon="arrow_back", on_click=go_back).props("flat round").tooltip("返回历史会话")
        ui.label("导出会话").classes("text-h6 font-semibold text-grey-9")

    # Main content - 完全垂直居中于整个视口
    with (
        ui.element("div")
        .classes("w-full bg-white")
        .style(
            "position: fixed; "
            "top: 0; "
            "left: 0; "
            "right: 0; "
            "bottom: 0; "
            "display: flex; "
            "align-items: center; "
            "justify-content: center; "
            "padding: 60px 80px; "
            "padding-top: calc(108px + 30px); "
            "box-sizing: border-box; "
            "overflow: auto;"
        ),
        ui.element("div").style(
            "display: flex; gap: 60px; align-items: center; max-width: 100%; width: 100%;"
        ),
    ):
        # Left column
        with ui.column().classes("gap-8").style("width: 360px; flex-shrink: 0;"):
            # Title
            with ui.column().classes("gap-2 mb-6"):
                candidate_words = [":)", ":D", "🚀", "🎉", "😄", "👍"]
                ui.label(random.choice(candidate_words)).classes("text-h3 font-bold text-grey-9")
                ui.label("选择导出方式").classes("text-h2 gradient-text").style("line-height: 1.2;")

            # Stats
            with ui.card().classes("glass-card p-5 w-full").style("margin-top: 40px;"):
                ui.label("统计").classes("text-subtitle2 font-semibold mb-4 text-grey-8")

                with ui.column().classes("gap-3 w-full"):
                    with ui.row().classes("items-center justify-between w-full"):
                        ui.label("会话总数").classes("text-xs text-grey-7")
                        ui.label(str(len(sessions))).classes("text-body1 font-bold text-primary")

                    with ui.row().classes("items-center justify-between w-full"):
                        ui.label("服务器状态").classes("text-xs text-grey-7")
                        status_label = ui.label("未启动").classes(
                            "text-body1 font-bold text-grey-6"
                        )

        # Right column - 2x2 grid
        with ui.element("div").style(
            "flex: 1; "
            "display: grid; "
            "grid-template-columns: repeat(2, 1fr); "
            "gap: 20px; "
            "max-width: 850px; "
            "margin-left: auto;"
        ):
            # Card 1: Server export
            server_card = (
                ui.card()
                .classes("super-card cursor-pointer")
                .style(
                    "height: 220px; "
                    "display: flex; "
                    "flex-direction: column; "
                    "padding: 1.5rem; "
                    "box-sizing: border-box;"
                )
            )

            with server_card:
                server_icon = (
                    ui.icon("cloud", size="3em")
                    .classes("text-primary rotate-icon")
                    .style("flex-shrink: 0; margin-bottom: 0.75rem;")
                )
                with ui.column().classes("gap-2").style("flex: 1;"):
                    ui.label("服务器导出").classes("text-h6 font-bold text-grey-9")
                    ui.label("通过浏览器访问导出页面").classes("text-sm text-grey-7")
                server_btn = (
                    ui.button("启动", icon="arrow_forward")
                    .props("color=primary flat dense")
                    .classes("press-button")
                    .style(
                        "align-self: flex-end; "
                        "flex-shrink: 0; "
                        "background: transparent ! important; "
                        "box-shadow: none !important;"
                    )
                )

            async def start_server_export():
                """Start or stop server export."""
                if server_running["value"]:
                    with (
                        ui.dialog() as stop_dialog,
                        ui.card()
                        .classes("dialog-card")
                        .style("width: 450px; padding: 28px; border-radius: 16px;"),
                    ):
                        with ui.row().classes("w-full items-center mb-6"):
                            ui.icon("warning", size="xl").classes("text-warning")
                            ui.label("确认关闭").classes(
                                "text-h5 font-bold text-grey-9 ml-3 flex-1"
                            )
                            ui.button(icon="close", on_click=stop_dialog.close).props(
                                "flat round dense"
                            )

                        ui.label("确定要停止服务器吗？").classes("text-body1 text-grey-8 mb-2")
                        ui.label("其他设备将无法继续访问").classes("text-body2 text-grey-7 mb-6")

                        with ui.row().classes("w-full gap-3 justify-end"):
                            ui.button("取消", on_click=stop_dialog.close).props(
                                "outline color=grey-8 no-caps"
                            ).classes("action-button")

                            async def confirm_stop():
                                await export_service.stop_server()
                                server_running["value"] = False
                                await ui.run_javascript("window.serverRunning = false;")
                                status_label.text = "未启动"
                                status_label.classes(remove="text-positive", add="text-grey-6")

                                # reset card UI
                                server_card.classes(remove="super-card-active")
                                server_icon.classes(remove="text-negative", add="text-primary")
                                server_btn.set_text("启动")
                                server_btn.props("icon=arrow_forward color=primary")

                                stop_dialog.close()
                                ui.notify("服务器已停止", type="info", position="top")

                            ui.button("确认停止", on_click=confirm_stop).props(
                                "unelevated color=negative no-caps"
                            ).classes("action-button")

                    stop_dialog.open()
                    return

                # Start server
                try:
                    info = await export_service.start_server()
                    server_running["value"] = True
                    await ui.run_javascript("window.serverRunning = true;")
                    status_label.text = "运行中"
                    status_label.classes(remove="text-grey-6", add="text-positive")

                    server_card.classes(add="super-card-active")
                    server_icon.classes(remove="text-primary", add="text-negative")
                    server_btn.set_text("停止")
                    server_btn.props("icon=stop color=negative")

                    # Show success dialog
                    with (
                        ui.dialog() as success_dialog,
                        ui.card()
                        .classes("dialog-card")
                        .style("width: 550px; padding: 32px; border-radius: 16px;"),
                    ):
                        with ui.row().classes("w-full items-center mb-6"):
                            ui.icon("check_circle", size="xl").classes("text-positive")
                            ui.label("服务器已启动").classes(
                                "text-h5 font-bold text-grey-9 q-ml-sm"
                            )
                            ui.space()
                            ui.button(icon="close", on_click=success_dialog.close).props(
                                "flat round dense"
                            )

                        # Server info
                        with ui.element("div").classes("info-box"):
                            ui.label("访问地址").classes("text-xs text-grey-6 mb-2")

                            with ui.row().classes("items-center gap-2 mb-3"):
                                ui.label(info.url).classes("text-body1 font-mono text-primary")

                                async def copy_url():
                                    await ui.run_javascript(
                                        f"navigator.clipboard.writeText('{info.url}')"
                                    )
                                    ui.notify("已复制", type="positive", position="top")

                                ui.button(icon="content_copy", on_click=copy_url).props(
                                    "flat dense round size=sm"
                                )

                            ui.element("div").classes("divider")

                            ui.label(f"主机: {info.host}").classes("text-sm text-grey-7 mb-1")
                            ui.label(f"端口: {info.port}").classes("text-sm text-grey-7")

                        ui.label("在其他设备的浏览器中打开上述地址即可访问").classes(
                            "text-body2 text-grey-6 mt-4 mb-6"
                        )

                        ui.button("知道了", on_click=success_dialog.close).props(
                            "unelevated color=primary no-caps"
                        ).classes("action-button w-full").style("height: 48px;")

                    success_dialog.open()

                except Exception:
                    ui.notify("启动失败", type="negative", position="top")

            server_card.on("click", start_server_export)

            # Card 2: USB export
            async def start_usb_export():
                """Start USB export."""
                devices = await export_service.list_usb_devices()

                if not devices:
                    with (
                        ui.dialog() as no_device_dialog,
                        ui.card()
                        .classes("dialog-card")
                        .style("width: 500px; padding: 32px; border-radius: 16px;"),
                    ):
                        with ui.row().classes("w-full items-center mb-6"):
                            ui.icon("usb_off", size="xl").classes("text-grey-4")
                            ui.label("未检测到U盘").classes("text-h5 font-bold text-grey-9 q-ml-sm")
                            ui.space()
                            ui.button(icon="close", on_click=no_device_dialog.close).props(
                                "flat round dense"
                            )

                        with ui.column().classes("items-center py-4 gap-3"):
                            ui.label("请插入U盘后重试").classes("text-body1 text-grey-7")

                            async def refresh():
                                no_device_dialog.close()
                                await start_usb_export()

                            ui.button("刷新", icon="refresh", on_click=refresh).props(
                                "flat no-caps"
                            ).classes("action-button mt-4")

                    no_device_dialog.open()
                    return

                device = devices[0]

                with (
                    ui.dialog() as usb_dialog,
                    ui.card()
                    .classes("dialog-card")
                    .style("width: 550px; padding: 32px; border-radius: 16px;"),
                ):
                    with ui.row().classes("w-full items-center mb-6"):
                        ui.icon("check_circle", size="xl").classes("text-positive")
                        ui.label("检测到U盘").classes("text-h5 font-bold text-grey-9 q-ml-sm")
                        ui.space()
                        ui.button(icon="close", on_click=usb_dialog.close).props("flat round dense")

                    with ui.element("div").classes("info-box"):
                        with ui.row().classes("items-center gap-3 mb-3"):
                            ui.icon("usb", size="lg").classes("text-secondary")
                            ui.label(device.label or "未命名设备").classes(
                                "text-body1 font-semibold"
                            )

                        if device.vendor or device.model:
                            ui.label(f"{device.vendor or ''} {device.model or ''}").classes(
                                "text-sm text-grey-6 mb-2"
                            )

                        ui.element("div").classes("divider")

                        ui.label(f"挂载点: {device.mount_point}").classes(
                            "text-sm text-grey-7 mb-1"
                        )
                        if device.size_bytes:
                            size_gb = device.size_bytes / (1024**3)
                            ui.label(f"容量: {size_gb:.2f} GB").classes("text-sm text-grey-7")

                    with (
                        ui.element("div").classes("export-summary-box mt-4"),
                        ui.row().classes("items-center justify-between"),
                    ):
                        ui.label("会话数量").classes("text-sm text-grey-6")
                        ui.label(f"{len(sessions)} 个").classes("text-body1 font-semibold")

                    async def do_export():
                        export_btn.props("loading")

                        try:
                            session_ids = [s.id for s in sessions]
                            result = await export_service.export_to_usb(session_ids, device)

                            usb_dialog.close()

                            if result.success:
                                ui.notify(
                                    f"成功导出 {result.success_count} 个会话",
                                    type="positive",
                                    position="top",
                                )
                            else:
                                ui.notify(
                                    f"部分失败: 成功 {result.success_count}/{result.total}",
                                    type="warning",
                                    position="top",
                                )

                        except Exception:
                            ui.notify("导出失败", type="negative", position="top")
                        finally:
                            export_btn.props(remove="loading")

                    export_btn = (
                        ui.button("开始导出", on_click=do_export)
                        .props("unelevated color=primary size=lg no-caps")
                        .classes("action-button w-full mt-6")
                        .style("height: 48px;")
                    )

                usb_dialog.open()

            with (
                ui.card()
                .classes("super-card cursor-pointer")
                .on("click", start_usb_export)
                .style(
                    "height: 220px; "
                    "display: flex; "
                    "flex-direction: column; "
                    "padding: 1.5rem; "
                    "box-sizing: border-box;"
                )
            ):
                ui.icon("usb", size="3em").classes("text-secondary rotate-icon").style(
                    "flex-shrink: 0; margin-bottom: 0.75rem;"
                )
                with ui.column().classes("gap-2").style("flex: 1;"):
                    ui.label("U盘导出").classes("text-h6 font-bold text-grey-9")
                    ui.label("直接导出到U盘设备").classes("text-sm text-grey-7")
                ui.button("开始", icon="arrow_forward").props("color=secondary flat dense").classes(
                    "press-button"
                ).style(
                    "align-self: flex-end; "
                    "flex-shrink: 0; "
                    "background: transparent !important; "
                    "box-shadow: none !important;"
                )



from __future__ import annotations

from audex.helper.settings import BaseModel
from audex.helper.settings.fields import Field
from audex.utils import UNSET
from audex.utils import Unset


class DashscopeCredentialConfig(BaseModel):
    api_key: str | Unset = Field(
        default=UNSET,
        description="Dashscope API key for authentication.",
    )



from __future__ import annotations

import json
import os
import pathlib
import typing as t

from pydantic import BaseModel as PydBaseModel
from pydantic import ConfigDict
from pydantic_core import PydanticUndefined
from pydantic_settings import BaseSettings

from audex import utils
from audex.exceptions import RequiredModuleNotFoundError


class BaseModel(PydBaseModel):
    """Base model class with common configuration for all models.

    This class sets default configurations for Pydantic models used in
    the Audex project, ensuring consistent behavior across all derived
    models.
    """

    model_config: t.ClassVar[ConfigDict] = ConfigDict(
        validate_assignment=True,
        extra="ignore",
        arbitrary_types_allowed=True,
        use_enum_values=True,
        populate_by_name=True,
    )

    def __hash__(self) -> int:
        """Generate a hash based on the model's serializable
        representation.

        Returns:
            An integer hash value.
        """
        serl_dict = self.model_dump()
        serl_json = json.dumps(serl_dict, sort_keys=True)
        return hash(serl_json)

    def __repr__(self) -> str:
        field_reprs = ", ".join(
            f"{field_name}={getattr(self, field_name)!r}"
            for field_name in self.model_fields  # type: ignore
        )
        return f"MODEL <{self.__class__.__name__}({field_reprs})>"


class Settings(BaseSettings):
    """Base settings class with YAML/JSON/dotenv export capabilities."""

    # ============================================================================
    # Public Methods - Loading
    # ============================================================================

    @classmethod
    def from_yaml(cls, path: str | pathlib.Path | os.PathLike[str]) -> t.Self:
        """Load configuration from YAML file.

        Args:
            path: Path to YAML configuration file.

        Returns:
            Settings instance.

        Raises:
            RequiredModuleNotFoundError: If PyYAML is not installed.
        """
        try:
            import yaml

            data = yaml.safe_load(pathlib.Path(path).read_text(encoding="utf-8"))
            return cls.model_validate(data, strict=False)
        except ImportError as e:
            raise RequiredModuleNotFoundError(
                "`yaml` module is required to load configuration from YAML "
                "files. Please install it using `pip install pyyaml`."
            ) from e

    # ============================================================================
    # Public Methods - Export
    # ============================================================================

    def serl(self, include_none: bool = False) -> dict[str, t.Any]:
        """Get a serializable version of the model dump.

        Args:
            include_none: If True, include fields with None values.

        Returns:
            Dictionary with only serializable values.
        """
        raw_dump = self.model_dump()
        return self._clean_dict(raw_dump, include_none=include_none)

    def to_yaml(
        self,
        fpath: str | pathlib.Path | os.PathLike[str],
        exclude_unset: bool = False,
        exclude_none: bool = False,
        with_comments: bool = True,
    ) -> None:
        """Export configuration to YAML file with optional field
        descriptions as comments.

        Args:
            fpath: Path to the output YAML file.
            exclude_unset: If True, exclude fields with Unset values and empty nested models.
            exclude_none: If True, exclude fields with None values.
            with_comments: If True, include field descriptions as comments.
        """
        descriptions = self._collect_field_desc() if with_comments else {}
        data = self.serl(include_none=True)

        # Clean unset and none values
        if exclude_unset:
            data = self._clean_unset_recursive(data)
        if exclude_none:
            data = self._clean_none_recursive(data)

        def write_yaml_value(
            f: t.TextIO,
            key: str,
            value: t.Any,
            field_path: str,
            indent: int = 0,
        ) -> None:
            """Write a single YAML key-value pair with proper
            formatting."""
            indent_str = "  " * indent
            desc = descriptions.get(field_path, "")
            comment = f" # {desc}" if desc and with_comments else ""

            if value is None:
                f.write(f"{indent_str}{key}: ~{comment}\n")
            elif isinstance(value, dict):
                f.write(f"{indent_str}{key}:{comment}\n")
                write_yaml_dict(f, value, field_path, indent + 1)
            elif isinstance(value, list):
                f.write(f"{indent_str}{key}:{comment}\n")
                write_yaml_list(f, value, field_path, indent + 1)
            elif isinstance(value, str) and ("\n" in value or len(value) > 80):
                f.write(f"{indent_str}{key}: |-{comment}\n")
                for line in value.split("\n"):
                    f.write(f"{indent_str}  {line}\n")
            else:
                yaml_value = self._yaml_repr(value)
                f.write(f"{indent_str}{key}: {yaml_value}{comment}\n")

        def write_yaml_dict(
            f: t.TextIO,
            obj: dict[str, t.Any],
            prefix: str,
            indent: int = 0,
        ) -> None:
            """Write a dictionary as YAML."""
            for key, value in obj.items():
                field_path = f"{prefix}.{key}" if prefix else key
                write_yaml_value(f, key, value, field_path, indent)

        def write_yaml_list(
            f: t.TextIO,
            items: list[t.Any],
            prefix: str,
            indent: int = 0,
        ) -> None:
            """Write a list as YAML."""
            indent_str = "  " * indent
            for item in items:
                if isinstance(item, dict):
                    items_list = list(item.items())
                    if items_list:
                        first_key, first_value = items_list[0]
                        first_field_path = f"{prefix}.{first_key}"
                        first_desc = descriptions.get(first_field_path, "")
                        first_comment = f"  # {first_desc}" if first_desc and with_comments else ""

                        if first_value is None:
                            f.write(f"{indent_str}- {first_key}: ~{first_comment}\n")
                        elif isinstance(first_value, dict):
                            f.write(f"{indent_str}- {first_key}:{first_comment}\n")
                            write_yaml_dict(f, first_value, first_field_path, indent + 1)
                        elif isinstance(first_value, list):
                            f.write(f"{indent_str}- {first_key}:{first_comment}\n")
                            write_yaml_list(f, first_value, first_field_path, indent + 1)
                        elif isinstance(first_value, str) and (
                            "\n" in first_value or len(first_value) > 80
                        ):
                            f.write(f"{indent_str}- {first_key}: |-{first_comment}\n")
                            for line in first_value.split("\n"):
                                f.write(f"{indent_str}    {line}\n")
                        else:
                            yaml_val = self._yaml_repr(first_value)
                            f.write(f"{indent_str}- {first_key}: {yaml_val}{first_comment}\n")

                        for sub_key, sub_value in items_list[1:]:
                            sub_field_path = f"{prefix}.{sub_key}"
                            write_yaml_value(f, sub_key, sub_value, sub_field_path, indent + 1)
                elif isinstance(item, list):
                    f.write(f"{indent_str}-\n")
                    write_yaml_list(f, item, prefix, indent + 1)
                else:
                    yaml_item = self._yaml_repr(item)
                    f.write(f"{indent_str}- {yaml_item}\n")

        with pathlib.Path(fpath).open("w", encoding="utf-8") as f:
            write_yaml_dict(f, data, "")

    def to_system_yaml(
        self,
        fpath: str | pathlib.Path | os.PathLike[str],
        platform: t.Literal["linux", "windows"] | None = None,
    ) -> None:
        """Export system configuration to YAML file with platform-
        specific defaults.

        This method generates a system configuration file using platform-specific
        default values defined in AudexFieldInfo descriptors. Fields with Unset
        values are omitted. Empty nested models (all fields Unset) are also omitted.

        Args:
            fpath: Path to the output YAML file
            platform: Target platform ("linux" or "windows"). If None, uses current platform.
        """
        if platform is None:
            platform = "linux" if os.name != "nt" else "windows"

        # Build data with platform-specific defaults
        data = self._build_platform_data(self.__class__, platform)

        # Clean Unset values and empty dicts
        data = self._clean_unset_values(data)

        # Write YAML without comments
        with pathlib.Path(fpath).open("w", encoding="utf-8") as f:
            self._write_system_yaml_header(f, platform)
            self._write_yaml_dict(f, data)

    def to_dotenv(self, fpath: str | pathlib.Path | os.PathLike[str]) -> None:
        """Export configuration to .env file with field descriptions as
        comments.

        Args:
            fpath: Path to the output .env file.
        """
        sep = self.model_config.get("env_nested_delimiter") or "__"
        prefix = self.model_config.get("env_prefix") or "AUDEX__"

        if prefix.endswith(sep):
            prefix = prefix[: -len(sep)]

        serializable_data = self.serl(include_none=True)
        descriptions = self._collect_field_desc()

        grouped_keys = {}
        for top_level_key in serializable_data:
            section_data = {top_level_key: serializable_data[top_level_key]}
            flattened = utils.flatten_dict(section_data, sep=sep)
            grouped_keys[top_level_key] = flattened

        with pathlib.Path(fpath).open("w", encoding="utf-8") as f:
            f.write(
                "# Description: Example environment configuration file for Audex application.\n"
            )
            f.write("# Note: Copy this file to '.env' and modify the values as needed.\n\n")
            for top_key, flattened in grouped_keys.items():
                f.write(f"# {'=' * 70}\n")

                top_key_desc = descriptions.get(top_key)
                if top_key_desc:
                    f.write(f"# {top_key.upper()}: {top_key_desc}\n")
                f.write(f"# {'=' * 70}\n")
                f.write("\n")

                for key, value in flattened.items():
                    env_key = f"{prefix}{sep}{key.upper()}"

                    field_path = key.replace(sep, ".")
                    if field_path in descriptions:
                        f.write(f"# {descriptions[field_path]}\n")

                    if value is None:
                        f.write(f"# {env_key}=\n")
                    else:
                        formatted_value = self._format_env_value(value)
                        f.write(f"{env_key}={formatted_value}\n")
                    f.write("\n")

    # ============================================================================
    # Private Methods - Field Introspection
    # ============================================================================

    def _collect_field_desc(
        self,
        model: type[BaseModel] | None = None,
        prefix: str = "",
    ) -> dict[str, str]:
        """Recursively collect field descriptions from the model and
        nested models.

        Args:
            model: The pydantic model to collect descriptions from.
            prefix: The prefix for nested field paths.

        Returns:
            A dictionary mapping field paths to their descriptions.
        """
        if model is None:
            model = self.__class__

        descriptions = {}
        for field_name, field_info in model.model_fields.items():
            field_path = f"{prefix}.{field_name}" if prefix else field_name

            if field_info.description:
                descriptions[field_path] = field_info.description

            if hasattr(field_info.annotation, "model_fields"):
                nested_descriptions = self._collect_field_desc(
                    field_info.annotation, prefix=field_path
                )
                descriptions.update(nested_descriptions)

        return descriptions

    # ============================================================================
    # Private Methods - Serialization
    # ============================================================================

    def _serl_value(self, value: t.Any, include_none: bool = False) -> t.Any:
        """Serialize a value, handling special cases like callables.

        Args:
            value: The value to serialize.
            include_none: If True, preserve None values in nested structures.

        Returns:
            Serialized value or None if not serializable.
        """
        if value is None:
            return value if include_none else None

        if isinstance(value, os.PathLike):
            try:
                return os.fspath(value)
            except Exception:
                return None

        if callable(value):
            try:
                return self._serl_value(value(), include_none=include_none)
            except Exception:
                return None

        if isinstance(value, PydBaseModel):
            return value.model_dump()

        serialized: list[t.Any] | dict[str, t.Any]

        if isinstance(value, (list, tuple)):
            serialized = []
            for item in value:
                serialized_item = self._serl_value(item, include_none=include_none)
                if include_none or serialized_item is not None:
                    serialized.append(serialized_item)
            return serialized if serialized or include_none else None

        if isinstance(value, dict):
            serialized = {}
            for k, v in value.items():
                serialized_v = self._serl_value(v, include_none=include_none)
                if include_none or serialized_v is not None:
                    serialized[k] = serialized_v
            return serialized if serialized or include_none else None

        if isinstance(value, (str, int, float, bool)):
            return value

        try:
            json.dumps(value)
            return value
        except (TypeError, ValueError):
            return None

    def _clean_dict(self, data: dict[str, t.Any], include_none: bool = False) -> dict[str, t.Any]:
        """Recursively clean a dictionary to remove non-serializable
        values.

        Args:
            data: Dictionary to clean.
            include_none: If True, include fields with None values.

        Returns:
            Cleaned dictionary.
        """
        cleaned: dict[str, t.Any] = {}
        for key, value in data.items():
            if value is None:
                if include_none:
                    cleaned[key] = None
            else:
                serialized = self._serl_value(value, include_none=include_none)
                if serialized is not None or include_none:
                    cleaned[key] = serialized
        return cleaned

    def _clean_unset_recursive(self, data: t.Any) -> t.Any:
        """Recursively remove Unset values and empty nested structures.

        Args:
            data: Data structure to clean.

        Returns:
            Cleaned data structure with Unset values and empty dicts removed.
        """
        if isinstance(data, utils.Unset) or isinstance(data, str) and data == "<UNSET>":
            return None

        if isinstance(data, dict):
            cleaned = {}
            for key, value in data.items():
                # Skip Unset values
                if isinstance(value, utils.Unset) or isinstance(value, str) and value == "<UNSET>":
                    continue

                cleaned_value = self._clean_unset_recursive(value)

                # Skip None (which indicates Unset was found)
                if cleaned_value is None:
                    continue

                # Skip empty dicts (all nested fields were Unset)
                if isinstance(cleaned_value, dict) and not cleaned_value:
                    continue

                # Skip empty lists
                if isinstance(cleaned_value, list) and not cleaned_value:
                    continue

                cleaned[key] = cleaned_value

            # Return None if dict is empty (all fields were Unset)
            return cleaned if cleaned else None

        if isinstance(data, list):
            cleaned_list = []
            for item in data:
                if isinstance(item, utils.Unset) or isinstance(item, str) and item == "<UNSET>":
                    continue

                cleaned_item = self._clean_unset_recursive(item)
                if cleaned_item is not None:
                    # Don't add empty dicts or lists
                    if isinstance(cleaned_item, (dict, list)) and not cleaned_item:
                        continue
                    cleaned_list.append(cleaned_item)

            return cleaned_list if cleaned_list else None

        return data

    def _clean_none_recursive(self, data: t.Any) -> t.Any:
        """Recursively remove None values and empty nested structures.

        Args:
            data: Data structure to clean.

        Returns:
            Cleaned data structure with None values removed.
        """
        if data is None:
            return None

        if isinstance(data, dict):
            cleaned = {}
            for key, value in data.items():
                if value is None:
                    continue

                cleaned_value = self._clean_none_recursive(value)

                if cleaned_value is None:
                    continue

                # Skip empty dicts
                if isinstance(cleaned_value, dict) and not cleaned_value:
                    continue

                # Skip empty lists
                if isinstance(cleaned_value, list) and not cleaned_value:
                    continue

                cleaned[key] = cleaned_value

            return cleaned if cleaned else None

        if isinstance(data, list):
            cleaned_list = []
            for item in data:
                if item is None:
                    continue

                cleaned_item = self._clean_none_recursive(item)
                if cleaned_item is not None:
                    if isinstance(cleaned_item, (dict, list)) and not cleaned_item:
                        continue
                    cleaned_list.append(cleaned_item)

            return cleaned_list if cleaned_list else None

        return data

    # ============================================================================
    # Private Methods - System YAML Generation
    # ============================================================================

    def _build_platform_data(
        self,
        model: type[PydBaseModel],
        platform: str,
        prefix: str = "",
    ) -> dict[str, t.Any]:
        """Recursively build configuration data with platform-specific
        defaults.

        Args:
            model: Pydantic model to process.
            platform: Target platform.
            prefix: Field path prefix.

        Returns:
            Dictionary with platform-specific default values.
        """
        from audex.helper.settings.fields import AudexFieldInfo

        result = {}

        for field_name, field_info in model.model_fields.items():
            field_path = f"{prefix}.{field_name}" if prefix else field_name

            # Handle nested models
            if hasattr(field_info.annotation, "model_fields"):
                nested_data = self._build_platform_data(
                    field_info.annotation,  # type: ignore
                    platform,
                    field_path,
                )
                # Only add if nested model has content
                if nested_data:
                    result[field_name] = nested_data
                continue

            # Get value based on field type
            if isinstance(field_info, AudexFieldInfo):
                value = field_info.get_platform_default(platform)
            else:
                # Standard field - use current value or default
                current_value = getattr(self, field_name, PydanticUndefined)
                if current_value is not PydanticUndefined:
                    value = current_value
                elif field_info.default is not PydanticUndefined:
                    value = field_info.default
                elif field_info.default_factory is not None:
                    value = field_info.default_factory()
                else:
                    value = PydanticUndefined

            # Serialize the value (handle Pydantic models, lists, dicts, etc.)
            if value is not PydanticUndefined:
                serialized_value = self._serl_value(value, include_none=True)
                result[field_name] = serialized_value
            else:
                result[field_name] = value

        return result

    def _clean_unset_values(self, data: t.Any) -> t.Any:
        """Recursively remove Unset values, PydanticUndefined, and empty
        dictionaries.

        Args:
            data: Data structure to clean.

        Returns:
            Cleaned data structure, or None if all values were Unset.
        """
        # Check for Unset or PydanticUndefined
        if isinstance(data, utils.Unset) or data is PydanticUndefined:
            return None

        if isinstance(data, dict):
            cleaned = {}
            for key, value in data.items():
                # Skip Unset and PydanticUndefined
                if isinstance(value, utils.Unset) or value is PydanticUndefined:
                    continue

                cleaned_value = self._clean_unset_values(value)

                # Skip None values from recursive cleaning
                if cleaned_value is None:
                    continue

                # Skip empty dicts (all fields were Unset)
                if isinstance(cleaned_value, dict) and not cleaned_value:
                    continue

                cleaned[key] = cleaned_value

            return cleaned if cleaned else None

        if isinstance(data, list):
            cleaned_list = []
            for item in data:
                if isinstance(item, utils.Unset) or item is PydanticUndefined:
                    continue

                cleaned_item = self._clean_unset_values(item)
                if cleaned_item is not None:
                    cleaned_list.append(cleaned_item)

            return cleaned_list if cleaned_list else None

        return data

    def _write_system_yaml_header(self, f: t.TextIO, platform: str) -> None:
        """Write system YAML file header.

        Args:
            f: File object.
            platform: Target platform.
        """
        import audex

        f.write("# Audex System Configuration\n")
        f.write(f"# Platform: {platform}\n")
        f.write(f"# Version: {audex.__version__}\n")
        f.write("#\n")
        f.write("# This file is generated automatically. Do not edit manually.\n")
        f.write("# User configuration should be placed in ~/.config/audex/config.yml\n")
        f.write("#\n")
        f.write("# For configuration examples, see: /etc/audex/config.example.yml\n")
        f.write("\n")

    def _write_yaml_dict(
        self,
        f: t.TextIO,
        obj: dict[str, t.Any],
        indent: int = 0,
    ) -> None:
        """Write a dictionary as YAML without comments.

        Args:
            f: File object.
            obj: Dictionary to write.
            indent: Indentation level.
        """
        for key, value in obj.items():
            self._write_yaml_value(f, key, value, indent)

    def _write_yaml_value(
        self,
        f: t.TextIO,
        key: str,
        value: t.Any,
        indent: int = 0,
    ) -> None:
        """Write a single YAML key-value pair without comments.

        Args:
            f: File object.
            key: Key name.
            value: Value to write.
            indent: Indentation level.
        """
        indent_str = "  " * indent

        if value is None:
            f.write(f"{indent_str}{key}: ~\n")
        elif isinstance(value, dict):
            f.write(f"{indent_str}{key}:\n")
            self._write_yaml_dict(f, value, indent + 1)
        elif isinstance(value, list):
            f.write(f"{indent_str}{key}:\n")
            self._write_yaml_list(f, value, indent + 1)
        elif isinstance(value, str) and ("\n" in value or len(value) > 80):
            f.write(f"{indent_str}{key}: |-\n")
            for line in value.split("\n"):
                f.write(f"{indent_str}  {line}\n")
        else:
            yaml_value = self._yaml_repr(value)
            f.write(f"{indent_str}{key}: {yaml_value}\n")

    def _write_yaml_list(
        self,
        f: t.TextIO,
        items: list[t.Any],
        indent: int = 0,
    ) -> None:
        """Write a list as YAML without comments.

        Args:
            f: File object.
            items: List items to write.
            indent: Indentation level.
        """
        indent_str = "  " * indent
        for item in items:
            if isinstance(item, dict):
                items_list = list(item.items())
                if items_list:
                    first_key, first_value = items_list[0]

                    if first_value is None:
                        f.write(f"{indent_str}- {first_key}: ~\n")
                    elif isinstance(first_value, dict):
                        f.write(f"{indent_str}- {first_key}:\n")
                        self._write_yaml_dict(f, first_value, indent + 1)
                    elif isinstance(first_value, list):
                        f.write(f"{indent_str}- {first_key}:\n")
                        self._write_yaml_list(f, first_value, indent + 1)
                    elif isinstance(first_value, str) and (
                        "\n" in first_value or len(first_value) > 80
                    ):
                        f.write(f"{indent_str}- {first_key}: |-\n")
                        for line in first_value.split("\n"):
                            f.write(f"{indent_str}    {line}\n")
                    else:
                        yaml_val = self._yaml_repr(first_value)
                        f.write(f"{indent_str}- {first_key}: {yaml_val}\n")

                    for sub_key, sub_value in items_list[1:]:
                        self._write_yaml_value(f, sub_key, sub_value, indent + 1)
            elif isinstance(item, list):
                f.write(f"{indent_str}-\n")
                self._write_yaml_list(f, item, indent + 1)
            else:
                yaml_item = self._yaml_repr(item)
                f.write(f"{indent_str}- {yaml_item}\n")

    # ============================================================================
    # Private Methods - YAML Formatting
    # ============================================================================

    def _yaml_repr(self, value: t.Any) -> str:
        """Convert a Python value to YAML representation string.

        Args:
            value: The value to convert.

        Returns:
            YAML string representation.
        """
        if value is None:
            return "~"
        if isinstance(value, bool):
            return "true" if value else "false"
        if isinstance(value, (int, float)):
            return str(value)
        if isinstance(value, str):
            if (
                not value
                or value[0] in "-? :,[]{}#&*! |>'\"%@`"
                or value in ("true", "false", "null", "yes", "no", "on", "off")
                or ":" in value
                or "#" in value
            ):
                escaped = value.replace("\\", "\\\\").replace('"', '\\"').replace("\n", "\\n")
                return f'"{escaped}"'
            return value
        return str(value)

    # ============================================================================
    # Private Methods - Dotenv Formatting
    # ============================================================================

    def _format_env_value(self, value: t.Any) -> str:
        """Format a value for .env file with proper quoting.

        Args:
            value: The value to format.

        Returns:
            Formatted string value.
        """
        if value is None:
            return ""

        if isinstance(value, os.PathLike):
            try:
                return os.fspath(value)
            except Exception:
                return ""

        if isinstance(value, bool):
            return "true" if value else "false"

        if isinstance(value, (int, float)):
            return str(value)

        if isinstance(value, str):
            needs_quotes = (
                " " in value
                or "#" in value
                or "=" in value
                or "\n" in value
                or "\t" in value
                or "\r" in value
                or value.startswith(('"', "'"))
                or not value
            )

            if needs_quotes:
                escaped = value.replace("\\", "\\\\").replace('"', '\\"').replace("\n", "\\n")
                return f'"{escaped}"'
            return value

        if isinstance(value, (list, dict)):
            json_str = json.dumps(value, separators=(",", ":"), ensure_ascii=False)
            escaped = json_str.replace("\\", "\\\\").replace('"', '\\"')
            return f'"{escaped}"'

        str_value = str(value)
        if " " in str_value or "#" in str_value:
            escaped = str_value.replace("\\", "\\\\").replace('"', '\\"')
            return f'"{escaped}"'
        return str_value



from __future__ import annotations

from audex import __prog__
from audex.helper.settings import BaseModel
from audex.helper.settings.fields import Field


class AudioConfig(BaseModel):
    sample_rate: int = Field(
        default=16000,
        description="The sample rate of the audio in Hz.",
    )

    vpr_sample_rate: int = Field(
        default=16000,
        description="The sample rate for voice print recognition in Hz.",
    )

    vpr_text_content: str = Field(
        default="您好，请问您今天需要什么帮助？",
        min_length=10,
        max_length=100,
        description="The text content used for voice print recognition.",
    )

    vpr_threshold: float = Field(
        default=0.5,
        gt=0.0,
        lt=1.0,
        description="The threshold for voice print recognition similarity.",
    )

    key_prefix: str = Field(
        default=__prog__,
        min_length=1,
        max_length=50,
        description="The prefix for storing audio files in cloud storage.",
    )

    segment_buffer: int = Field(
        default=2000,
        gt=0,
        lt=5000,
        description="The buffer time in milliseconds for audio segmentation.",
    )



# This file is auto-generated by PrototypeX filter generator.
# Do not edit manually - changes will be overwritten.
# Regenerate using: python -m scripts.gen_filters gen

from __future__ import annotations

import datetime
import typing as t

from audex.entity.doctor import Doctor
from audex.filters import FieldFilter
from audex.filters import Filter
from audex.filters import FilterBuilder
from audex.filters import StringBackedFieldFilter
from audex.filters import StringFieldFilter
from audex.valueobj.common.auth import HashedPassword
from audex.valueobj.common.email import Email
from audex.valueobj.common.phone import CNPhone


class DoctorFilterBuilder(FilterBuilder[Doctor]):
    """Doctor filter builder with full type hints and IDE support.

    This class is auto-generated. Do not edit manually.
    """

    class _IdField(StringFieldFilter):
        """Chainable id filter with type-safe operations."""

        def __init__(self, builder: DoctorFilterBuilder) -> None:
            super().__init__("id", object.__getattribute__(builder, "_filter"))
            self._parent_builder = builder

        def eq(self, value: str) -> DoctorFilterBuilder:
            super().eq(value)
            return self._parent_builder

        def ne(self, value: str) -> DoctorFilterBuilder:
            super().ne(value)
            return self._parent_builder

        def gt(self, value: str) -> DoctorFilterBuilder:
            super().gt(value)
            return self._parent_builder

        def lt(self, value: str) -> DoctorFilterBuilder:
            super().lt(value)
            return self._parent_builder

        def gte(self, value: str) -> DoctorFilterBuilder:
            super().gte(value)
            return self._parent_builder

        def lte(self, value: str) -> DoctorFilterBuilder:
            super().lte(value)
            return self._parent_builder

        def in_(self, values: t.Sequence[str]) -> DoctorFilterBuilder:
            super().in_(values)
            return self._parent_builder

        def nin(self, values: t.Sequence[str]) -> DoctorFilterBuilder:
            super().nin(values)
            return self._parent_builder

        def between(self, value1: str, value2: str) -> DoctorFilterBuilder:
            super().between(value1, value2)
            return self._parent_builder

        def is_null(self) -> DoctorFilterBuilder:
            super().is_null()
            return self._parent_builder

        def is_not_null(self) -> DoctorFilterBuilder:
            super().is_not_null()
            return self._parent_builder

        def asc(self) -> DoctorFilterBuilder:
            super().asc()
            return self._parent_builder

        def desc(self) -> DoctorFilterBuilder:
            super().desc()
            return self._parent_builder

        def contains(self, value: str) -> DoctorFilterBuilder:
            super().contains(value)
            return self._parent_builder

        def startswith(self, value: str) -> DoctorFilterBuilder:
            super().startswith(value)
            return self._parent_builder

        def endswith(self, value: str) -> DoctorFilterBuilder:
            super().endswith(value)
            return self._parent_builder

    class _CreatedAtField(FieldFilter[datetime.datetime]):
        """Chainable created_at filter with type-safe operations."""

        def __init__(self, builder: DoctorFilterBuilder) -> None:
            super().__init__("created_at", object.__getattribute__(builder, "_filter"))
            self._parent_builder = builder

        def eq(self, value: datetime.datetime) -> DoctorFilterBuilder:
            super().eq(value)
            return self._parent_builder

        def ne(self, value: datetime.datetime) -> DoctorFilterBuilder:
            super().ne(value)
            return self._parent_builder

        def gt(self, value: datetime.datetime) -> DoctorFilterBuilder:
            super().gt(value)
            return self._parent_builder

        def lt(self, value: datetime.datetime) -> DoctorFilterBuilder:
            super().lt(value)
            return self._parent_builder

        def gte(self, value: datetime.datetime) -> DoctorFilterBuilder:
            super().gte(value)
            return self._parent_builder

        def lte(self, value: datetime.datetime) -> DoctorFilterBuilder:
            super().lte(value)
            return self._parent_builder

        def in_(self, values: t.Sequence[datetime.datetime]) -> DoctorFilterBuilder:
            super().in_(values)
            return self._parent_builder

        def nin(self, values: t.Sequence[datetime.datetime]) -> DoctorFilterBuilder:
            super().nin(values)
            return self._parent_builder

        def between(
            self, value1: datetime.datetime, value2: datetime.datetime
        ) -> DoctorFilterBuilder:
            super().between(value1, value2)
            return self._parent_builder

        def is_null(self) -> DoctorFilterBuilder:
            super().is_null()
            return self._parent_builder

        def is_not_null(self) -> DoctorFilterBuilder:
            super().is_not_null()
            return self._parent_builder

        def asc(self) -> DoctorFilterBuilder:
            super().asc()
            return self._parent_builder

        def desc(self) -> DoctorFilterBuilder:
            super().desc()
            return self._parent_builder

    class _UpdatedAtField(FieldFilter[datetime.datetime | None]):
        """Chainable updated_at filter with type-safe operations."""

        def __init__(self, builder: DoctorFilterBuilder) -> None:
            super().__init__("updated_at", object.__getattribute__(builder, "_filter"))
            self._parent_builder = builder

        def eq(self, value: datetime.datetime | None) -> DoctorFilterBuilder:
            super().eq(value)
            return self._parent_builder

        def ne(self, value: datetime.datetime | None) -> DoctorFilterBuilder:
            super().ne(value)
            return self._parent_builder

        def gt(self, value: datetime.datetime | None) -> DoctorFilterBuilder:
            super().gt(value)
            return self._parent_builder

        def lt(self, value: datetime.datetime | None) -> DoctorFilterBuilder:
            super().lt(value)
            return self._parent_builder

        def gte(self, value: datetime.datetime | None) -> DoctorFilterBuilder:
            super().gte(value)
            return self._parent_builder

        def lte(self, value: datetime.datetime | None) -> DoctorFilterBuilder:
            super().lte(value)
            return self._parent_builder

        def in_(self, values: t.Sequence[datetime.datetime | None]) -> DoctorFilterBuilder:
            super().in_(values)
            return self._parent_builder

        def nin(self, values: t.Sequence[datetime.datetime | None]) -> DoctorFilterBuilder:
            super().nin(values)
            return self._parent_builder

        def between(
            self, value1: datetime.datetime | None, value2: datetime.datetime | None
        ) -> DoctorFilterBuilder:
            super().between(value1, value2)
            return self._parent_builder

        def is_null(self) -> DoctorFilterBuilder:
            super().is_null()
            return self._parent_builder

        def is_not_null(self) -> DoctorFilterBuilder:
            super().is_not_null()
            return self._parent_builder

        def asc(self) -> DoctorFilterBuilder:
            super().asc()
            return self._parent_builder

        def desc(self) -> DoctorFilterBuilder:
            super().desc()
            return self._parent_builder

    class _EidField(StringFieldFilter):
        """Chainable eid filter with type-safe operations."""

        def __init__(self, builder: DoctorFilterBuilder) -> None:
            super().__init__("eid", object.__getattribute__(builder, "_filter"))
            self._parent_builder = builder

        def eq(self, value: str) -> DoctorFilterBuilder:
            super().eq(value)
            return self._parent_builder

        def ne(self, value: str) -> DoctorFilterBuilder:
            super().ne(value)
            return self._parent_builder

        def gt(self, value: str) -> DoctorFilterBuilder:
            super().gt(value)
            return self._parent_builder

        def lt(self, value: str) -> DoctorFilterBuilder:
            super().lt(value)
            return self._parent_builder

        def gte(self, value: str) -> DoctorFilterBuilder:
            super().gte(value)
            return self._parent_builder

        def lte(self, value: str) -> DoctorFilterBuilder:
            super().lte(value)
            return self._parent_builder

        def in_(self, values: t.Sequence[str]) -> DoctorFilterBuilder:
            super().in_(values)
            return self._parent_builder

        def nin(self, values: t.Sequence[str]) -> DoctorFilterBuilder:
            super().nin(values)
            return self._parent_builder

        def between(self, value1: str, value2: str) -> DoctorFilterBuilder:
            super().between(value1, value2)
            return self._parent_builder

        def is_null(self) -> DoctorFilterBuilder:
            super().is_null()
            return self._parent_builder

        def is_not_null(self) -> DoctorFilterBuilder:
            super().is_not_null()
            return self._parent_builder

        def asc(self) -> DoctorFilterBuilder:
            super().asc()
            return self._parent_builder

        def desc(self) -> DoctorFilterBuilder:
            super().desc()
            return self._parent_builder

        def contains(self, value: str) -> DoctorFilterBuilder:
            super().contains(value)
            return self._parent_builder

        def startswith(self, value: str) -> DoctorFilterBuilder:
            super().startswith(value)
            return self._parent_builder

        def endswith(self, value: str) -> DoctorFilterBuilder:
            super().endswith(value)
            return self._parent_builder

    class _PasswordHashField(StringBackedFieldFilter[HashedPassword]):
        """Chainable password_hash filter with type-safe operations."""

        def __init__(self, builder: DoctorFilterBuilder) -> None:
            super().__init__("password_hash", object.__getattribute__(builder, "_filter"))
            self._parent_builder = builder

        def eq(self, value: HashedPassword) -> DoctorFilterBuilder:
            super().eq(value)
            return self._parent_builder

        def ne(self, value: HashedPassword) -> DoctorFilterBuilder:
            super().ne(value)
            return self._parent_builder

        def gt(self, value: HashedPassword) -> DoctorFilterBuilder:
            super().gt(value)
            return self._parent_builder

        def lt(self, value: HashedPassword) -> DoctorFilterBuilder:
            super().lt(value)
            return self._parent_builder

        def gte(self, value: HashedPassword) -> DoctorFilterBuilder:
            super().gte(value)
            return self._parent_builder

        def lte(self, value: HashedPassword) -> DoctorFilterBuilder:
            super().lte(value)
            return self._parent_builder

        def in_(self, values: t.Sequence[HashedPassword]) -> DoctorFilterBuilder:
            super().in_(values)
            return self._parent_builder

        def nin(self, values: t.Sequence[HashedPassword]) -> DoctorFilterBuilder:
            super().nin(values)
            return self._parent_builder

        def between(self, value1: HashedPassword, value2: HashedPassword) -> DoctorFilterBuilder:
            super().between(value1, value2)
            return self._parent_builder

        def is_null(self) -> DoctorFilterBuilder:
            super().is_null()
            return self._parent_builder

        def is_not_null(self) -> DoctorFilterBuilder:
            super().is_not_null()
            return self._parent_builder

        def asc(self) -> DoctorFilterBuilder:
            super().asc()
            return self._parent_builder

        def desc(self) -> DoctorFilterBuilder:
            super().desc()
            return self._parent_builder

        def contains(self, value: str) -> DoctorFilterBuilder:
            super().contains(value)
            return self._parent_builder

        def startswith(self, value: str) -> DoctorFilterBuilder:
            super().startswith(value)
            return self._parent_builder

        def endswith(self, value: str) -> DoctorFilterBuilder:
            super().endswith(value)
            return self._parent_builder

    class _NameField(StringFieldFilter):
        """Chainable name filter with type-safe operations."""

        def __init__(self, builder: DoctorFilterBuilder) -> None:
            super().__init__("name", object.__getattribute__(builder, "_filter"))
            self._parent_builder = builder

        def eq(self, value: str) -> DoctorFilterBuilder:
            super().eq(value)
            return self._parent_builder

        def ne(self, value: str) -> DoctorFilterBuilder:
            super().ne(value)
            return self._parent_builder

        def gt(self, value: str) -> DoctorFilterBuilder:
            super().gt(value)
            return self._parent_builder

        def lt(self, value: str) -> DoctorFilterBuilder:
            super().lt(value)
            return self._parent_builder

        def gte(self, value: str) -> DoctorFilterBuilder:
            super().gte(value)
            return self._parent_builder

        def lte(self, value: str) -> DoctorFilterBuilder:
            super().lte(value)
            return self._parent_builder

        def in_(self, values: t.Sequence[str]) -> DoctorFilterBuilder:
            super().in_(values)
            return self._parent_builder

        def nin(self, values: t.Sequence[str]) -> DoctorFilterBuilder:
            super().nin(values)
            return self._parent_builder

        def between(self, value1: str, value2: str) -> DoctorFilterBuilder:
            super().between(value1, value2)
            return self._parent_builder

        def is_null(self) -> DoctorFilterBuilder:
            super().is_null()
            return self._parent_builder

        def is_not_null(self) -> DoctorFilterBuilder:
            super().is_not_null()
            return self._parent_builder

        def asc(self) -> DoctorFilterBuilder:
            super().asc()
            return self._parent_builder

        def desc(self) -> DoctorFilterBuilder:
            super().desc()
            return self._parent_builder

        def contains(self, value: str) -> DoctorFilterBuilder:
            super().contains(value)
            return self._parent_builder

        def startswith(self, value: str) -> DoctorFilterBuilder:
            super().startswith(value)
            return self._parent_builder

        def endswith(self, value: str) -> DoctorFilterBuilder:
            super().endswith(value)
            return self._parent_builder

    class _DepartmentField(StringFieldFilter):
        """Chainable department filter with type-safe operations."""

        def __init__(self, builder: DoctorFilterBuilder) -> None:
            super().__init__("department", object.__getattribute__(builder, "_filter"))
            self._parent_builder = builder

        def eq(self, value: str | None) -> DoctorFilterBuilder:
            super().eq(value)
            return self._parent_builder

        def ne(self, value: str | None) -> DoctorFilterBuilder:
            super().ne(value)
            return self._parent_builder

        def gt(self, value: str | None) -> DoctorFilterBuilder:
            super().gt(value)
            return self._parent_builder

        def lt(self, value: str | None) -> DoctorFilterBuilder:
            super().lt(value)
            return self._parent_builder

        def gte(self, value: str | None) -> DoctorFilterBuilder:
            super().gte(value)
            return self._parent_builder

        def lte(self, value: str | None) -> DoctorFilterBuilder:
            super().lte(value)
            return self._parent_builder

        def in_(self, values: t.Sequence[str | None]) -> DoctorFilterBuilder:
            super().in_(values)
            return self._parent_builder

        def nin(self, values: t.Sequence[str | None]) -> DoctorFilterBuilder:
            super().nin(values)
            return self._parent_builder

        def between(self, value1: str | None, value2: str | None) -> DoctorFilterBuilder:
            super().between(value1, value2)
            return self._parent_builder

        def is_null(self) -> DoctorFilterBuilder:
            super().is_null()
            return self._parent_builder

        def is_not_null(self) -> DoctorFilterBuilder:
            super().is_not_null()
            return self._parent_builder

        def asc(self) -> DoctorFilterBuilder:
            super().asc()
            return self._parent_builder

        def desc(self) -> DoctorFilterBuilder:
            super().desc()
            return self._parent_builder

        def contains(self, value: str) -> DoctorFilterBuilder:
            super().contains(value)
            return self._parent_builder

        def startswith(self, value: str) -> DoctorFilterBuilder:
            super().startswith(value)
            return self._parent_builder

        def endswith(self, value: str) -> DoctorFilterBuilder:
            super().endswith(value)
            return self._parent_builder

    class _TitleField(StringFieldFilter):
        """Chainable title filter with type-safe operations."""

        def __init__(self, builder: DoctorFilterBuilder) -> None:
            super().__init__("title", object.__getattribute__(builder, "_filter"))
            self._parent_builder = builder

        def eq(self, value: str | None) -> DoctorFilterBuilder:
            super().eq(value)
            return self._parent_builder

        def ne(self, value: str | None) -> DoctorFilterBuilder:
            super().ne(value)
            return self._parent_builder

        def gt(self, value: str | None) -> DoctorFilterBuilder:
            super().gt(value)
            return self._parent_builder

        def lt(self, value: str | None) -> DoctorFilterBuilder:
            super().lt(value)
            return self._parent_builder

        def gte(self, value: str | None) -> DoctorFilterBuilder:
            super().gte(value)
            return self._parent_builder

        def lte(self, value: str | None) -> DoctorFilterBuilder:
            super().lte(value)
            return self._parent_builder

        def in_(self, values: t.Sequence[str | None]) -> DoctorFilterBuilder:
            super().in_(values)
            return self._parent_builder

        def nin(self, values: t.Sequence[str | None]) -> DoctorFilterBuilder:
            super().nin(values)
            return self._parent_builder

        def between(self, value1: str | None, value2: str | None) -> DoctorFilterBuilder:
            super().between(value1, value2)
            return self._parent_builder

        def is_null(self) -> DoctorFilterBuilder:
            super().is_null()
            return self._parent_builder

        def is_not_null(self) -> DoctorFilterBuilder:
            super().is_not_null()
            return self._parent_builder

        def asc(self) -> DoctorFilterBuilder:
            super().asc()
            return self._parent_builder

        def desc(self) -> DoctorFilterBuilder:
            super().desc()
            return self._parent_builder

        def contains(self, value: str) -> DoctorFilterBuilder:
            super().contains(value)
            return self._parent_builder

        def startswith(self, value: str) -> DoctorFilterBuilder:
            super().startswith(value)
            return self._parent_builder

        def endswith(self, value: str) -> DoctorFilterBuilder:
            super().endswith(value)
            return self._parent_builder

    class _HospitalField(StringFieldFilter):
        """Chainable hospital filter with type-safe operations."""

        def __init__(self, builder: DoctorFilterBuilder) -> None:
            super().__init__("hospital", object.__getattribute__(builder, "_filter"))
            self._parent_builder = builder

        def eq(self, value: str | None) -> DoctorFilterBuilder:
            super().eq(value)
            return self._parent_builder

        def ne(self, value: str | None) -> DoctorFilterBuilder:
            super().ne(value)
            return self._parent_builder

        def gt(self, value: str | None) -> DoctorFilterBuilder:
            super().gt(value)
            return self._parent_builder

        def lt(self, value: str | None) -> DoctorFilterBuilder:
            super().lt(value)
            return self._parent_builder

        def gte(self, value: str | None) -> DoctorFilterBuilder:
            super().gte(value)
            return self._parent_builder

        def lte(self, value: str | None) -> DoctorFilterBuilder:
            super().lte(value)
            return self._parent_builder

        def in_(self, values: t.Sequence[str | None]) -> DoctorFilterBuilder:
            super().in_(values)
            return self._parent_builder

        def nin(self, values: t.Sequence[str | None]) -> DoctorFilterBuilder:
            super().nin(values)
            return self._parent_builder

        def between(self, value1: str | None, value2: str | None) -> DoctorFilterBuilder:
            super().between(value1, value2)
            return self._parent_builder

        def is_null(self) -> DoctorFilterBuilder:
            super().is_null()
            return self._parent_builder

        def is_not_null(self) -> DoctorFilterBuilder:
            super().is_not_null()
            return self._parent_builder

        def asc(self) -> DoctorFilterBuilder:
            super().asc()
            return self._parent_builder

        def desc(self) -> DoctorFilterBuilder:
            super().desc()
            return self._parent_builder

        def contains(self, value: str) -> DoctorFilterBuilder:
            super().contains(value)
            return self._parent_builder

        def startswith(self, value: str) -> DoctorFilterBuilder:
            super().startswith(value)
            return self._parent_builder

        def endswith(self, value: str) -> DoctorFilterBuilder:
            super().endswith(value)
            return self._parent_builder

    class _PhoneField(StringFieldFilter):
        """Chainable phone filter with type-safe operations."""

        def __init__(self, builder: DoctorFilterBuilder) -> None:
            super().__init__("phone", object.__getattribute__(builder, "_filter"))
            self._parent_builder = builder

        def eq(self, value: CNPhone | None) -> DoctorFilterBuilder:
            super().eq(value)
            return self._parent_builder

        def ne(self, value: CNPhone | None) -> DoctorFilterBuilder:
            super().ne(value)
            return self._parent_builder

        def gt(self, value: CNPhone | None) -> DoctorFilterBuilder:
            super().gt(value)
            return self._parent_builder

        def lt(self, value: CNPhone | None) -> DoctorFilterBuilder:
            super().lt(value)
            return self._parent_builder

        def gte(self, value: CNPhone | None) -> DoctorFilterBuilder:
            super().gte(value)
            return self._parent_builder

        def lte(self, value: CNPhone | None) -> DoctorFilterBuilder:
            super().lte(value)
            return self._parent_builder

        def in_(self, values: t.Sequence[CNPhone | None]) -> DoctorFilterBuilder:
            super().in_(values)
            return self._parent_builder

        def nin(self, values: t.Sequence[CNPhone | None]) -> DoctorFilterBuilder:
            super().nin(values)
            return self._parent_builder

        def between(self, value1: CNPhone | None, value2: CNPhone | None) -> DoctorFilterBuilder:
            super().between(value1, value2)
            return self._parent_builder

        def is_null(self) -> DoctorFilterBuilder:
            super().is_null()
            return self._parent_builder

        def is_not_null(self) -> DoctorFilterBuilder:
            super().is_not_null()
            return self._parent_builder

        def asc(self) -> DoctorFilterBuilder:
            super().asc()
            return self._parent_builder

        def desc(self) -> DoctorFilterBuilder:
            super().desc()
            return self._parent_builder

        def contains(self, value: str) -> DoctorFilterBuilder:
            super().contains(value)
            return self._parent_builder

        def startswith(self, value: str) -> DoctorFilterBuilder:
            super().startswith(value)
            return self._parent_builder

        def endswith(self, value: str) -> DoctorFilterBuilder:
            super().endswith(value)
            return self._parent_builder

    class _EmailField(StringBackedFieldFilter[Email | None]):
        """Chainable email filter with type-safe operations."""

        def __init__(self, builder: DoctorFilterBuilder) -> None:
            super().__init__("email", object.__getattribute__(builder, "_filter"))
            self._parent_builder = builder

        def eq(self, value: Email | None) -> DoctorFilterBuilder:
            super().eq(value)
            return self._parent_builder

        def ne(self, value: Email | None) -> DoctorFilterBuilder:
            super().ne(value)
            return self._parent_builder

        def gt(self, value: Email | None) -> DoctorFilterBuilder:
            super().gt(value)
            return self._parent_builder

        def lt(self, value: Email | None) -> DoctorFilterBuilder:
            super().lt(value)
            return self._parent_builder

        def gte(self, value: Email | None) -> DoctorFilterBuilder:
            super().gte(value)
            return self._parent_builder

        def lte(self, value: Email | None) -> DoctorFilterBuilder:
            super().lte(value)
            return self._parent_builder

        def in_(self, values: t.Sequence[Email | None]) -> DoctorFilterBuilder:
            super().in_(values)
            return self._parent_builder

        def nin(self, values: t.Sequence[Email | None]) -> DoctorFilterBuilder:
            super().nin(values)
            return self._parent_builder

        def between(self, value1: Email | None, value2: Email | None) -> DoctorFilterBuilder:
            super().between(value1, value2)
            return self._parent_builder

        def is_null(self) -> DoctorFilterBuilder:
            super().is_null()
            return self._parent_builder

        def is_not_null(self) -> DoctorFilterBuilder:
            super().is_not_null()
            return self._parent_builder

        def asc(self) -> DoctorFilterBuilder:
            super().asc()
            return self._parent_builder

        def desc(self) -> DoctorFilterBuilder:
            super().desc()
            return self._parent_builder

        def contains(self, value: str) -> DoctorFilterBuilder:
            super().contains(value)
            return self._parent_builder

        def startswith(self, value: str) -> DoctorFilterBuilder:
            super().startswith(value)
            return self._parent_builder

        def endswith(self, value: str) -> DoctorFilterBuilder:
            super().endswith(value)
            return self._parent_builder

    class _IsActiveField(FieldFilter[bool]):
        """Chainable is_active filter with type-safe operations."""

        def __init__(self, builder: DoctorFilterBuilder) -> None:
            super().__init__("is_active", object.__getattribute__(builder, "_filter"))
            self._parent_builder = builder

        def eq(self, value: bool) -> DoctorFilterBuilder:
            super().eq(value)
            return self._parent_builder

        def ne(self, value: bool) -> DoctorFilterBuilder:
            super().ne(value)
            return self._parent_builder

        def gt(self, value: bool) -> DoctorFilterBuilder:
            super().gt(value)
            return self._parent_builder

        def lt(self, value: bool) -> DoctorFilterBuilder:
            super().lt(value)
            return self._parent_builder

        def gte(self, value: bool) -> DoctorFilterBuilder:
            super().gte(value)
            return self._parent_builder

        def lte(self, value: bool) -> DoctorFilterBuilder:
            super().lte(value)
            return self._parent_builder

        def in_(self, values: t.Sequence[bool]) -> DoctorFilterBuilder:
            super().in_(values)
            return self._parent_builder

        def nin(self, values: t.Sequence[bool]) -> DoctorFilterBuilder:
            super().nin(values)
            return self._parent_builder

        def between(self, value1: bool, value2: bool) -> DoctorFilterBuilder:
            super().between(value1, value2)
            return self._parent_builder

        def is_null(self) -> DoctorFilterBuilder:
            super().is_null()
            return self._parent_builder

        def is_not_null(self) -> DoctorFilterBuilder:
            super().is_not_null()
            return self._parent_builder

        def asc(self) -> DoctorFilterBuilder:
            super().asc()
            return self._parent_builder

        def desc(self) -> DoctorFilterBuilder:
            super().desc()
            return self._parent_builder

    def and_(self, *filters: Filter) -> Filter:
        """Combine with other filters using AND logic.

        Args:
            *filters: Other filters to combine with AND.

        Returns:
            Combined filter with AND logic.

        Example:
            ```python
            # username = 'john' AND email = 'john@ex.com'
            filter = (
                doctor_filter()
                .username.eq("john")
                .and_(doctor_filter().email.eq("john@ex.com"))
            )
            ```
        """
        return object.__getattribute__(self, "_filter").and_(*filters)

    def or_(self, *filters: Filter) -> Filter:
        """Combine with other filters using OR logic.

        Args:
            *filters: Other filters to combine with OR.

        Returns:
            Combined filter with OR logic.

        Example:
            ```python
            # username = 'john' OR email = 'john@ex.com'
            filter = (
                doctor_filter()
                .username.eq("john")
                .or_(doctor_filter().email.eq("john@ex.com"))
            )
            ```
        """
        return object.__getattribute__(self, "_filter").or_(*filters)

    def not_(self) -> Filter:
        """Negate the current filter.

        Returns:
            Negated filter.

        Example:
            ```python
            # NOT (username = 'john')
            filter = doctor_filter().username.eq("john").not_()
            ```
        """
        return object.__getattribute__(self, "_filter").not_()

    # Field properties

    @property
    def id(self) -> _IdField:
        """Filter by id field."""
        return self._IdField(self)

    @property
    def created_at(self) -> _CreatedAtField:
        """Filter by created_at field."""
        return self._CreatedAtField(self)

    @property
    def updated_at(self) -> _UpdatedAtField:
        """Filter by updated_at field."""
        return self._UpdatedAtField(self)

    @property
    def eid(self) -> _EidField:
        """Filter by eid field."""
        return self._EidField(self)

    @property
    def password_hash(self) -> _PasswordHashField:
        """Filter by password_hash field."""
        return self._PasswordHashField(self)

    @property
    def name(self) -> _NameField:
        """Filter by name field."""
        return self._NameField(self)

    @property
    def department(self) -> _DepartmentField:
        """Filter by department field."""
        return self._DepartmentField(self)

    @property
    def title(self) -> _TitleField:
        """Filter by title field."""
        return self._TitleField(self)

    @property
    def hospital(self) -> _HospitalField:
        """Filter by hospital field."""
        return self._HospitalField(self)

    @property
    def phone(self) -> _PhoneField:
        """Filter by phone field."""
        return self._PhoneField(self)

    @property
    def email(self) -> _EmailField:
        """Filter by email field."""
        return self._EmailField(self)

    @property
    def is_active(self) -> _IsActiveField:
        """Filter by is_active field."""
        return self._IsActiveField(self)


def doctor_filter() -> DoctorFilterBuilder:
    """Create a Doctor filter builder with full type safety.

    Returns:
        A DoctorFilterBuilder instance with full IDE support and chainable fields.

    Example:
        ```python
        # AND conditions (chained)
        filter = (
            doctor_filter()
            .field1.eq(value1)
            .field2.contains(value2)
        )

        # OR conditions
        filter = doctor_filter().field1.eq(
            value1
        ) | doctor_filter().field2.eq(value2)
        ```
    """
    return DoctorFilterBuilder(Doctor)



from __future__ import annotations

__title__ = "Audex"
__prog__ = "audex"
__version__ = "1.0.5-a"
__description__ = """
Audex is an intelligent medical transcription system designed specifically for clinical scenarios.
It uses Voice Print Recognition (VPR) and real-time speech-to-text technology to automatically distinguish between the doctor and the patient, producing structured medical documentation from their conversations.
"""



from __future__ import annotations

import contextlib
import os
import pathlib
import typing as t

from pydantic import model_validator

from audex.config.helper.client.http import HttpClientConfig
from audex.config.helper.provider.unisound import UnisoundCredentialConfig
from audex.config.helper.provider.xfyun import XFYunCredentialConfig
from audex.helper.settings import BaseModel
from audex.helper.settings.fields import Field


class XFYunVPRConfig(BaseModel):
    credential: XFYunCredentialConfig = Field(
        default_factory=XFYunCredentialConfig,
        description="Credentials for XFYun VPR service.",
    )

    group_id: str | None = Field(
        default=None,
        description="The group ID for voiceprint recognition.",
    )

    group_id_path: os.PathLike[str] = Field(
        default=pathlib.Path(".xf.vprgroup"),
        description="The file path to read the group ID from if not provided directly.",
        system_default=".xf.vprgroup",
        system_path_type="config",
    )

    http: HttpClientConfig = Field(
        default_factory=HttpClientConfig,
        description="HTTP client configuration for XFYun VPR service.",
    )

    @model_validator(mode="after")
    def load_group_id_from_file(self) -> t.Self:
        if self.group_id is None:
            with (
                contextlib.suppress(FileNotFoundError),
                pathlib.Path(self.group_id_path).open("r") as f,
            ):
                self.group_id = f.read().strip()
        return self


class UnisoundVPRConfig(BaseModel):
    credential: UnisoundCredentialConfig = Field(
        default_factory=UnisoundCredentialConfig,
        description="Credentials for Unisound VPR service.",
    )

    group_id: str | None = Field(
        default=None,
        description="The group ID for voiceprint recognition.",
    )

    group_id_path: os.PathLike[str] = Field(
        default=pathlib.Path(".unisound.vprgroup"),
        description="The file path to read the group ID from if not provided directly.",
        system_default=".unisound.vprgroup",
        system_path_type="config",
    )

    http: HttpClientConfig = Field(
        default_factory=HttpClientConfig,
        description="HTTP client configuration for Unisound VPR service.",
    )

    @model_validator(mode="after")
    def load_group_id_from_file(self) -> t.Self:
        if self.group_id is None:
            with (
                contextlib.suppress(FileNotFoundError),
                pathlib.Path(self.group_id_path).open("r") as f,
            ):
                self.group_id = f.read().strip()
        return self


class VPRConfig(BaseModel):
    provider: t.Literal["xfyun", "unisound"] = Field(
        default="xfyun",
        description="The VPR service provider to use.",
    )

    xfyun: XFYunVPRConfig = Field(
        default_factory=XFYunVPRConfig,
        description="Configuration for the XFYun VPR service.",
    )

    unisound: UnisoundVPRConfig = Field(
        default_factory=UnisoundVPRConfig,
        description="Configuration for the Unisound VPR service.",
    )



from __future__ import annotations


class ErrorMessages:
    """Error messages in Chinese for export service."""

    # Server errors
    SERVER_ALREADY_RUNNING = "服务器已在运行"
    SERVER_START_FAILED = "启动服务器失败"
    SERVER_STOP_FAILED = "停止服务器失败"

    # USB errors
    NO_USB_DEVICE = "未检测到U盘，请插入U盘后重试"
    USB_EXPORT_FAILED = "导出到U盘失败"

    # Session errors
    NO_ACTIVE_SESSION = "登录状态已过期，请重新登录"



from __future__ import annotations

import base64
import hashlib
import typing as t
import uuid

from httpx import URL
from httpx import AsyncClient

from audex import utils
from audex.lib.restful import RESTfulMixin
from audex.lib.vpr import VPR
from audex.lib.vpr import VPRError
from audex.lib.vpr.unisound.types import ConfirmFeatureRequest
from audex.lib.vpr.unisound.types import ConfirmFeatureResponse
from audex.lib.vpr.unisound.types import CreateFeatureRequest
from audex.lib.vpr.unisound.types import CreateFeatureResponse
from audex.lib.vpr.unisound.types import CreateGroupRequest
from audex.lib.vpr.unisound.types import CreateGroupResponse
from audex.lib.vpr.unisound.types import UpdateFeatureRequest
from audex.lib.vpr.unisound.types import UpdateFeatureResponse


class UnisoundVPR(RESTfulMixin, VPR):
    __logtag__ = "audex.lib.vpr.unisound"

    def __init__(
        self,
        *,
        appkey: str,
        secret: str,
        group_id: str | None = None,
        base_url: str = "https://ai-vpr.hivoice.cn",
        proxy: str | URL | None = None,
        timeout: float = 10.0,
        http_client: AsyncClient | None = None,
        default_headers: dict[str, str] | None = None,
        default_params: dict[str, t.Any] | None = None,
    ):
        self.appkey = appkey
        self.secret = secret
        self.group_id = group_id
        super().__init__(
            base_url=base_url,
            proxy=proxy,
            timeout=timeout,
            http_client=http_client,
            default_headers=default_headers,
            default_params=default_params,
        )

    def _generate_sign(self, timestamp: int, nonce: str) -> str:
        """Generate signature for authentication."""
        sign_str = f"{self.appkey}{timestamp}{self.secret}{nonce}"
        return hashlib.sha256(sign_str.encode("utf-8")).hexdigest().upper()

    def _build_auth_params(self) -> dict[str, t.Any]:
        """Build authentication parameters."""
        timestamp = int(utils.utcnow().timestamp() * 1000)
        nonce = uuid.uuid4().hex
        sign = self._generate_sign(timestamp, nonce)
        return {
            "appkey": self.appkey,
            "timestamp": timestamp,
            "nonce": nonce,
            "sign": sign,
        }

    async def create_group(self, name: str, gid: str | None = None) -> str:
        if self.group_id:
            raise VPRError("Group already exists, cannot create a new one.")
        group_id = gid or utils.gen_id()
        auth_params = self._build_auth_params()

        response = await self.request(
            endpoint="/vpr/v1/createGroup",
            method="POST",
            json=CreateGroupRequest(
                **auth_params,
                groupId=group_id,
                groupInfo=name,
            ).model_dump(by_alias=True),
            cast_to=CreateGroupResponse,
        )

        if response.code != 0:
            raise VPRError(f"Failed to create group: [{response.code}] {response.msg}")

        if not response.data or response.data.group_id != group_id:
            raise VPRError("Group ID mismatch after creation.")

        self.group_id = group_id
        return self.group_id

    async def enroll(self, data: bytes, sr: int, uid: str | None = None) -> str:
        if not self.group_id:
            raise VPRError("Group ID is not set. Cannot enroll feature.")

        uid = uid or utils.gen_id()
        auth_params = self._build_auth_params()

        audio_data = base64.b64encode(data).decode("utf-8")

        response = await self.request(
            endpoint="/vpr/v1/createFeature",
            method="POST",
            json=CreateFeatureRequest(
                **auth_params,
                groupId=self.group_id,
                featureId=uid,
                featureInfo=f"Feature {uid}",
                audioData=audio_data,
                audioSampleRate=sr,
                audioFormat="pcm",
            ).model_dump(by_alias=True),
            cast_to=CreateFeatureResponse,
        )

        if response.code != 0:
            raise VPRError(f"Failed to enroll feature: [{response.code}] {response.msg}")

        if not response.data or response.data.feature_id != uid:
            raise VPRError("Feature ID mismatch after registration.")

        return uid

    async def update(self, uid: str, data: bytes, sr: int) -> None:
        if not self.group_id:
            raise VPRError("Group ID is not set. Cannot update feature.")

        auth_params = self._build_auth_params()

        audio_data = base64.b64encode(data).decode("utf-8")

        response = await self.request(
            endpoint="/vpr/v1/updateFeatureById",
            method="POST",
            json=UpdateFeatureRequest(
                **auth_params,
                groupId=self.group_id,
                featureId=uid,
                featureInfo=f"Updated feature {uid}",
                audioData=audio_data,
                audioSampleRate=sr,
                audioFormat="pcm",
            ).model_dump(by_alias=True),
            cast_to=UpdateFeatureResponse,
        )

        if response.code != 0:
            raise VPRError(f"Failed to update feature: [{response.code}] {response.msg}")

        if not response.data or not response.data.result:
            raise VPRError("Feature update failed.")

    async def verify(self, uid: str, data: bytes, sr: int) -> float:
        if not self.group_id:
            raise VPRError("Group ID is not set. Cannot verify feature.")

        auth_params = self._build_auth_params()

        audio_data = base64.b64encode(data).decode("utf-8")

        response = await self.request(
            endpoint="/vpr/v1/confirmFeature",
            method="POST",
            json=ConfirmFeatureRequest(
                **auth_params,
                groupId=self.group_id,
                featureId=uid,
                audioData=audio_data,
                audioSampleRate=sr,
                audioFormat="pcm",
            ).model_dump(by_alias=True),
            cast_to=ConfirmFeatureResponse,
        )

        if response.code != 0:
            raise VPRError(f"Failed to verify feature: [{response.code}] {response.msg}")

        if not response.data:
            raise VPRError("No verification data returned.")

        return response.data.score  # type: ignore



# ruff: noqa: N802
from __future__ import annotations

import datetime
import typing as t

if t.TYPE_CHECKING:
    from audex.entity import Entity


T = t.TypeVar("T")


class FieldSpec(t.Generic[T]):
    """A descriptor for entity fields with validation and default value
    support.

    This descriptor provides type-safe field access with optional validation,
    default values, immutability, and nullable support.

    Attributes:
        default: The default value for the field.
        default_factory: A callable that returns the default value.
        nullable: Whether the field can be None.
        immutable: Whether the field can be modified after initial assignment.
        sortable: Whether the field supports sorting operations.
        name: The public name of the field (set by __set_name__).
        private_name: The private attribute name for storing the value.
        _field_type: Stored type information for stub generation.

    Args:
        default: Default value for the field.
        default_factory: Callable that returns a default value.
        nullable: Whether None is allowed as a value.
        immutable: Whether the field can be modified after being set.
        sortable: Whether the field supports sorting operations.
    """

    def __init__(
        self,
        *,
        default: T | None = None,
        default_factory: t.Callable[[], T] | None = None,
        nullable: bool = False,
        immutable: bool = False,
        sortable: bool = True,
    ) -> None:
        self.default = default
        self.default_factory = default_factory
        self.nullable = nullable
        self.immutable = immutable
        self.sortable = sortable
        self.name: str = ""
        self.private_name: str = ""
        self._field_type: type | None = None  # Store type information

    def __set_name__(self, owner: type, name: str) -> None:
        """Set the field name when the descriptor is assigned to a class
        attribute.

        Args:
            owner: The class that owns this descriptor.
            name: The name of the attribute.
        """
        self.name = name
        self.private_name = f"_field_{name}"

    @t.overload
    def __get__(self, obj: None, objtype: type[t.Any] | None = None) -> t.Self: ...
    @t.overload
    def __get__(self, obj: Entity, objtype: type[t.Any] | None = None) -> T: ...
    def __get__(self, obj: Entity | None, objtype: type[t.Any] | None = None) -> T | t.Self:
        """Get the field value."""
        if obj is None:
            return self

        if not hasattr(obj, self.private_name):
            if self.default_factory is not None:
                value = self.default_factory()
            elif self.default is not None:
                value = self.default
            elif self.nullable:
                value = None
            else:
                raise AttributeError(f"Field '{self.name}' has not been set")
            setattr(obj, self.private_name, value)

        return t.cast(T, getattr(obj, self.private_name))

    def __set__(self, obj: Entity, value: T) -> None:
        """Set the field value."""
        if hasattr(obj, self.private_name) and self.immutable:
            raise AttributeError(f"Field '{self.name}' is immutable")

        if value is None and not self.nullable:
            raise ValueError(f"Field '{self.name}' cannot be None")

        setattr(obj, self.private_name, value)

    def __delete__(self, obj: Entity) -> None:
        """Delete the field value."""
        if self.immutable:
            raise AttributeError(f"Field '{self.name}' is immutable")
        if hasattr(obj, self.private_name):
            delattr(obj, self.private_name)


# Modified factory functions to store type information


class StringBackedFieldSpec(FieldSpec[T]):
    """A field descriptor for values that are persisted as strings in
    the database."""


@t.overload
def StringBackedField(
    _field_type: type[T],
    *,
    default: T | None = None,
    default_factory: t.Callable[[], T] | None = None,
    nullable: bool = False,
    immutable: bool = False,
    sortable: bool = True,
) -> StringBackedFieldSpec[T]: ...
@t.overload
def StringBackedField(
    _field_type: None = None,
    *,
    default: t.Any | None = None,
    default_factory: t.Callable[[], t.Any] | None = None,
    nullable: bool = False,
    immutable: bool = False,
    sortable: bool = True,
) -> StringBackedFieldSpec[t.Any]: ...
def StringBackedField(
    _field_type: type[t.Any] | None = None,
    *,
    default: t.Any | None = None,
    default_factory: t.Callable[[], t.Any] | None = None,
    nullable: bool = False,
    immutable: bool = False,
    sortable: bool = True,
) -> t.Any:
    """Factory function to create a StringBackedFieldSpec."""
    field = StringBackedFieldSpec(
        default=default,
        default_factory=default_factory,
        nullable=nullable,
        immutable=immutable,
        sortable=sortable,
    )
    field._field_type = _field_type
    return field


class ListFieldSpec(FieldSpec[list[T]]):
    """A field descriptor for list/collection values."""

    def __init__(
        self,
        *,
        default: list[T] | None = None,
        default_factory: t.Callable[[], list[T]] | None = None,
        nullable: bool = False,
        immutable: bool = False,
        sortable: bool = False,
    ) -> None:
        super().__init__(
            default=default,
            default_factory=default_factory,
            nullable=nullable,
            immutable=immutable,
            sortable=sortable,
        )


@t.overload
def ListField(
    _item_type: type[T],
    *,
    default: list[T] | None = None,
    default_factory: t.Callable[[], list[T]] | None = None,
    nullable: bool = False,
    immutable: bool = False,
    sortable: bool = False,
) -> ListFieldSpec[T]: ...
@t.overload
def ListField(
    _item_type: None = None,
    *,
    default: list[t.Any] | None = None,
    default_factory: t.Callable[[], list[t.Any]] | None = None,
    nullable: bool = False,
    immutable: bool = False,
    sortable: bool = False,
) -> ListFieldSpec[t.Any]: ...
def ListField(
    _item_type: type[t.Any] | None = None,
    *,
    default: list[t.Any] | None = None,
    default_factory: t.Callable[[], list[t.Any]] | None = None,
    nullable: bool = False,
    immutable: bool = False,
    sortable: bool = False,
) -> ListFieldSpec[t.Any]:
    """Factory function to create a ListFieldSpec."""
    field = ListFieldSpec(
        default=default,
        default_factory=default_factory,
        nullable=nullable,
        immutable=immutable,
        sortable=sortable,
    )
    field._field_type = _item_type
    return field


class ForeignFieldSpec(FieldSpec[T]):
    """A field descriptor for foreign/complex type values."""

    def __init__(
        self,
        *,
        default: T | None = None,
        default_factory: t.Callable[[], T] | None = None,
        nullable: bool = False,
        immutable: bool = False,
        sortable: bool = False,
    ) -> None:
        super().__init__(
            default=default,
            default_factory=default_factory,
            nullable=nullable,
            immutable=immutable,
            sortable=sortable,
        )


@t.overload
def ForeignField(
    _field_type: type[T],
    *,
    default: T | None = None,
    default_factory: t.Callable[[], T] | None = None,
    nullable: bool = False,
    immutable: bool = False,
    sortable: bool = False,
) -> ForeignFieldSpec[T]: ...
@t.overload
def ForeignField(
    _field_type: None = None,
    *,
    default: t.Any | None = None,
    default_factory: t.Callable[[], t.Any] | None = None,
    nullable: bool = False,
    immutable: bool = False,
    sortable: bool = False,
) -> ForeignFieldSpec[t.Any]: ...
def ForeignField(
    _field_type: type[t.Any] | None = None,
    *,
    default: t.Any | None = None,
    default_factory: t.Callable[[], t.Any] | None = None,
    nullable: bool = False,
    immutable: bool = False,
    sortable: bool = False,
) -> t.Any:
    """Factory function to create a ForeignFieldSpec."""
    field = ForeignFieldSpec(
        default=default,
        default_factory=default_factory,
        nullable=nullable,
        immutable=immutable,
        sortable=sortable,
    )
    field._field_type = _field_type
    return field


# Keep all other field types as they were
class StringFieldSpec(FieldSpec[str]):
    """A field descriptor for string values."""


def StringField(
    *,
    default: str | None = None,
    default_factory: t.Callable[[], str] | None = None,
    nullable: bool = False,
    immutable: bool = False,
    sortable: bool = False,
) -> t.Any:
    return StringFieldSpec(
        default=default,
        default_factory=default_factory,
        nullable=nullable,
        immutable=immutable,
        sortable=sortable,
    )


class IntegerFieldSpec(FieldSpec[int]):
    """A field descriptor for integer values."""


def IntegerField(
    *,
    default: int | None = None,
    default_factory: t.Callable[[], int] | None = None,
    nullable: bool = False,
    immutable: bool = False,
    sortable: bool = True,
) -> t.Any:
    return IntegerFieldSpec(
        default=default,
        default_factory=default_factory,
        nullable=nullable,
        immutable=immutable,
        sortable=sortable,
    )


class FloatFieldSpec(FieldSpec[float]):
    """A field descriptor for float values."""


def FloatField(
    *,
    default: float | None = None,
    default_factory: t.Callable[[], float] | None = None,
    nullable: bool = False,
    immutable: bool = False,
    sortable: bool = True,
) -> t.Any:
    return FloatFieldSpec(
        default=default,
        default_factory=default_factory,
        nullable=nullable,
        immutable=immutable,
        sortable=sortable,
    )


class BoolFieldSpec(FieldSpec[bool]):
    """A field descriptor for boolean values."""


def BoolField(
    *,
    default: bool | None = None,
    default_factory: t.Callable[[], bool] | None = None,
    nullable: bool = False,
    immutable: bool = False,
    sortable: bool = True,
) -> t.Any:
    return BoolFieldSpec(
        default=default,
        default_factory=default_factory,
        nullable=nullable,
        immutable=immutable,
        sortable=sortable,
    )


class DateTimeFieldSpec(FieldSpec[datetime.datetime]):
    """A field descriptor for datetime values."""


def DateTimeField(
    *,
    default: datetime.datetime | None = None,
    default_factory: t.Callable[[], datetime.datetime] | None = None,
    nullable: bool = False,
    immutable: bool = False,
    sortable: bool = True,
) -> t.Any:
    return DateTimeFieldSpec(
        default=default,
        default_factory=default_factory,
        nullable=nullable,
        immutable=immutable,
        sortable=sortable,
    )


def Field(
    *,
    default: t.Any | None = None,
    default_factory: t.Callable[[], t.Any] | None = None,
    nullable: bool = False,
    immutable: bool = False,
    sortable: bool = False,
) -> t.Any:
    return FieldSpec(
        default=default,
        default_factory=default_factory,
        nullable=nullable,
        immutable=immutable,
        sortable=sortable,
    )



from __future__ import annotations

import base64
import json
import typing as t

from starlette.middleware.base import BaseHTTPMiddleware
from starlette.requests import Request
from starlette.responses import RedirectResponse
from starlette.responses import Response

from audex.helper.mixin import LoggingMixin
from audex.lib.repos.doctor import DoctorRepository
from audex.lib.server.types import DoctorSessionData

if t.TYPE_CHECKING:
    from starlette.types import ASGIApp


class AuthMiddleware(BaseHTTPMiddleware, LoggingMixin):
    """Authentication middleware using cookie-based sessions."""

    __logtag__ = "audex.lib.http.auth"

    COOKIE_NAME: t.ClassVar[str] = "audex_session"
    COOKIE_MAX_AGE: t.ClassVar[int] = 86400 * 7  # 7 days

    # Public routes that don't require auth
    PUBLIC_ROUTES: t.ClassVar[set[str]] = {"/login", "/api/login", "/static"}

    def __init__(self, app: ASGIApp, doctor_repo: DoctorRepository):
        super().__init__(app)
        self.doctor_repo = doctor_repo

    async def dispatch(
        self, request: Request, call_next: t.Callable[[Request], t.Awaitable[Response]]
    ) -> Response:
        """Process request with authentication check."""
        # Check if route is public
        if self._is_public_route(request.url.path):
            return await call_next(request)

        # Get session from cookie
        session_data = self._get_session_from_cookie(request)

        if not session_data:
            # Not authenticated, redirect to login
            if request.url.path.startswith("/api/"):
                return Response(
                    content=json.dumps({"error": "Unauthorized"}),
                    status_code=401,
                    media_type="application/json",
                )
            return RedirectResponse(url="/login", status_code=303)

        # Verify doctor still exists and is active
        doctor = await self.doctor_repo.read(session_data["doctor_id"])
        if not doctor or not doctor.is_active:
            # Session invalid, clear cookie
            response = RedirectResponse(url="/login", status_code=303)
            response.delete_cookie(self.COOKIE_NAME)
            return response

        # Attach session data to request state
        request.state.doctor_session = session_data

        return await call_next(request)

    def _is_public_route(self, path: str) -> bool:
        """Check if route is public."""
        return any(path.startswith(public) for public in self.PUBLIC_ROUTES)

    def _get_session_from_cookie(self, request: Request) -> DoctorSessionData | None:
        """Extract and decode session from cookie."""
        cookie_value = request.cookies.get(self.COOKIE_NAME)
        if not cookie_value:
            return None

        try:
            # Decode base64
            decoded = base64.b64decode(cookie_value).decode("utf-8")
            session_dict = json.loads(decoded)

            # Validate required fields
            if not all(k in session_dict for k in ["doctor_id", "eid", "doctor_name"]):
                return None

            return t.cast(DoctorSessionData, session_dict)

        except Exception as e:
            self.logger.warning(f"Failed to decode session cookie: {e}")
            return None

    @staticmethod
    def create_session_cookie(session_data: DoctorSessionData) -> str:
        """Create encoded session cookie value."""
        json_str = json.dumps(session_data)
        return base64.b64encode(json_str.encode("utf-8")).decode("utf-8")
